@@ -92,8 +92,8 @@ from lino.core.modeltools import resolve_model, UnresolvedModel
 #~ # from lino.modlib.notes.models import NoteType
 #~ from lino.modlib.countries.models import Country, City
 
-if settings.LINO.user_model:
-    User = resolve_model(settings.LINO.user_model,strict=True)
+#~ if settings.LINO.user_model:
+    #~ User = resolve_model(settings.LINO.user_model,strict=True)
 
 
 def is_valid_niss(national_id):
@@ -393,10 +393,10 @@ class Person(CpasPartner,contacts.Person,contacts.Born,Printable):
         blank=True,null=True,
         verbose_name=_("until"))
     
-    coach1 = models.ForeignKey(settings.LINO.user_model,
+    coach1 = dd.ForeignKey(settings.LINO.user_model,
         blank=True,null=True,
         verbose_name=_("Coach 1"),related_name='coached1')
-    coach2 = models.ForeignKey(settings.LINO.user_model,
+    coach2 = dd.ForeignKey(settings.LINO.user_model,
         blank=True,null=True,
         verbose_name=_("Coach 2"),related_name='coached2')
         
@@ -418,12 +418,12 @@ class Person(CpasPartner,contacts.Person,contacts.Born,Printable):
         #~ ,validators=[niss_validator]
         )
         
-    health_insurance = models.ForeignKey(settings.LINO.company_model,blank=True,null=True,
+    health_insurance = dd.ForeignKey(settings.LINO.company_model,blank=True,null=True,
         verbose_name=_("Health insurance"),related_name='health_insurance_for')
-    pharmacy = models.ForeignKey(settings.LINO.company_model,blank=True,null=True,
+    pharmacy = dd.ForeignKey(settings.LINO.company_model,blank=True,null=True,
         verbose_name=_("Pharmacy"),related_name='pharmacy_for')
     
-    nationality = models.ForeignKey('countries.Country',
+    nationality = dd.ForeignKey('countries.Country',
         blank=True,null=True,
         related_name='by_nationality',
         verbose_name=_("Nationality"))
@@ -585,21 +585,26 @@ class Person(CpasPartner,contacts.Person,contacts.Born,Printable):
         self.update_reminders()
         
     def update_reminders(self):
+        """
+        Creates or updates automatic tasks controlled directly by this Person.
+        """
         user = self.coach2 or self.coach1
         if user:
-            M = DurationUnits.months
-            update_reminder(1,self,user,
-              self.card_valid_until,
-              _("eID card expires in 2 months"),2,M)
-            update_reminder(2,self,user,
-              self.unavailable_until,
-              _("becomes available again in 1 month"),1,M)
-            update_reminder(3,self,user,
-              self.work_permit_suspended_until,
-              _("work permit suspension ends in 1 month"),1,M)
-            update_reminder(4,self,user,
-              self.coached_until,
-              _("coaching ends in 1 month"),1,M)
+            def f():
+                M = DurationUnits.months
+                update_reminder(1,self,user,
+                  self.card_valid_until,
+                  _("eID card expires in 2 months"),2,M)
+                update_reminder(2,self,user,
+                  self.unavailable_until,
+                  _("becomes available again in 1 month"),1,M)
+                update_reminder(3,self,user,
+                  self.work_permit_suspended_until,
+                  _("work permit suspension ends in 1 month"),1,M)
+                update_reminder(4,self,user,
+                  self.coached_until,
+                  _("coaching ends in 1 month"),1,M)
+            babel.run_with_language(user.language,f)
               
           
     #~ def get_auto_task_defaults(self,**kw):
@@ -1080,20 +1085,16 @@ class AllPersons(Partners):
     #~ debug_actions = True
     model = settings.LINO.person_model
     detail_layout = PersonDetail()
+    insert_layout = dd.FormLayout("""
+    title first_name last_name
+    gender language
+    """,window_size=(60,'auto'))
+    
     order_by = "last_name first_name id".split()
-    #~ can_view = perms.is_authenticated
-    #~ column_names = "name_column national_id gsm street street_no street_box city age email phone id bank_account1 aid_type coach1 language *"
-    #~ column_names = "name_column:20 national_id:10 gsm:10 address_column age:10 email phone:10 id bank_account1 aid_type coach1 language:10 *"
     column_names = "name_column:20 national_id:10 gsm:10 address_column age:10 email phone:10 id bank_account1 aid_type coach1 language:10"
     
     app_label = 'contacts'
     
-    #~ default_params = dict(is_active=True)
-    #~ extra = dict(
-      #~ select=dict(sort_name='lower(last_name||first_name)'),
-      #~ order_by=['sort_name'])
-    #~ order_by = None # clear the default value from contacts.Persons.order_by since we use extra order_by
-    
     
     @classmethod
     def get_actor_label(self):
@@ -1283,7 +1284,7 @@ class ClientsTest(Persons):
     #~ required_user_level = UserLevels.manager
     label = _("Data Test Clients")
     parameters = dict(
-      user = models.ForeignKey(User,blank=True,verbose_name=_("Coached by")),
+      user = dd.ForeignKey(settings.LINO.user_model,blank=True,verbose_name=_("Coached by")),
       today = models.DateField(_("only active on"),blank=True,default=datetime.date.today),
       invalid_niss = models.BooleanField(_("Check NISS validity"),default=True),
       overlapping_contracts = models.BooleanField(_("Check for overlapping contracts"),default=True),
@@ -1412,30 +1413,15 @@ class UsersWithClients(dd.VirtualTable):
         So we add the rule that only system admins see other system admins.
         
         """
-        #~ if ar.get_user().is_superuser:
-        #~ if ar.get_user().level >= UserLevels.expert:
-            #~ # flt = Q(is_spis=True) 
-            #~ flt = Q(integ_level__gt='') 
-        #~ else:
-            #~ # flt = Q(is_spis=True,is_superuser=False)
-            #~ # flt = Q(integ_level__isnull=False,is_superuser=False)
-            #~ flt = Q(integ_level__gt='',level__gte=UserLevels.expert)
-        #~ qs = User.objects.exclude(integ_level='')
         profiles = [p for p in dd.UserProfiles.items() if p.integ_level]
-        #~ profiles = [p for p in UserProfiles.items()]
-        #~ logger.info('20120629 %r', [x.value for x in profiles])
-        #~ logger.info('20120629 %r', profiles)
-        #~ (UserProfiles.melanie,UserProfiles.hubert,UserProfiles.admin)
-        qs = User.objects.filter(profile__in=profiles)
-        #~ qs = User.objects.all()
+        qs = users.User.objects.filter(profile__in=profiles)
         if ar.get_user().profile.level < UserLevels.admin:
-            #~ qs = qs.exclude(level__gte=UserLevels.manager)
             qs = qs.exclude(profile__gte=UserLevels.admin)
         for user in qs.order_by('username'):
             r = MyPersons.request(ar.ui,subst_user=user)
             if r.get_total_count():
-            #~ if len(r.data_iterator):
                 user.my_persons = r
+                #~ user._detail_action = users.MySettings.default_action
                 yield user
                 
     @dd.virtualfield('contacts.Person.coach1')
@@ -1648,7 +1634,7 @@ class PersonSearch(mixins.AutoUser,mixins.Printable):
     
     only_my_persons = models.BooleanField(_("Only my persons")) # ,default=True)
     
-    coached_by = models.ForeignKey(settings.LINO.user_model,
+    coached_by = dd.ForeignKey(settings.LINO.user_model,
         verbose_name=_("Coached by"),
         related_name='persons_coached',
         blank=True,null=True)
@@ -2121,7 +2107,7 @@ def site_setup(site):
     #~ """)
     site.modules.users.Users.set_detail_layout("""
     box1:50 box2:25
-    remarks AuthoritiesByUser
+    remarks AuthoritiesGiven 
     """,
     box2="""
     newcomer_quota
