@@ -36,16 +36,32 @@ from django.http import HttpResponse, HttpResponseRedirect, Http404
 from django.utils.encoding import force_unicode
 
 import lino
-from lino import dd
+#~ from lino import dd
 
 from lino.core import actions
 from lino.core import fields
-from lino.utils import iif
+from lino.utils import iif, moneyfmt
 from lino.utils import babel 
 #~ from lino.utils import call_optional_super
 from lino.utils.choosers import chooser
-from lino.utils.appy_pod import setup_renderer
-from lino.tools import makedirs_if_missing
+from lino.utils.appy_pod import Renderer
+from lino.utils.config import find_template_config_files
+from lino.core.modeltools import makedirs_if_missing
+from lino.core.modeltools import Model
+from lino.mixins.duplicable import Duplicable
+
+
+def decfmt(v,places=2,**kw):
+    """
+    Format a Decimal value.
+    Like :func:`lino.utils.moneyfmt`, but using the site settings
+    :attr:`lino.Lno.decimal_group_separator`
+    and
+    :attr:`lino.Lno.decimal_separator`.
+    """
+    kw.setdefault('sep',settings.LINO.decimal_group_separator)
+    kw.setdefault('dp',settings.LINO.decimal_separator)
+    return moneyfmt(v,places=places,**kw)
 
 
 
@@ -55,10 +71,10 @@ try:
 except ImportError:
     pisa = None
 
-try:
-    import appy
-except ImportError:
-    appy = None
+#~ try:
+    #~ import appy
+#~ except ImportError:
+    #~ appy = None
     
 try:
     import pyratemp
@@ -151,7 +167,7 @@ class BuildMethod:
         #~ return settings.MEDIA_URL + "/".join(self.get_target_parts(action,elem))
         return ui.media_url(*self.get_target_parts(action,elem))
             
-    def build(self,action,elem):
+    def build(self,ar,action,elem):
         raise NotImplementedError
         
     #~ def get_template_url(self,action,elem):
@@ -162,12 +178,12 @@ class DjangoBuildMethod(BuildMethod):
     def get_template(self,action,elem):
         tpls = action.get_print_templates(self,elem)
         if len(tpls) == 0:
-            raise Exception("No templates defined for %r" % elem)
+            raise actions.Warning("No templates defined for %r" % elem)
         #~ logger.debug('make_pisa_html %s',tpls)
         try:
             return select_template(tpls)
         except TemplateDoesNotExist,e:
-            raise Exception("No template found for %s (%s)" % (e,tpls))
+            raise actions.Warning("No template found for %s (%s)" % (e,tpls))
 
     def render_template(self,elem,tpl): # ,MEDIA_URL=settings.MEDIA_URL):
         context = dict(
@@ -186,7 +202,7 @@ class PisaBuildMethod(DjangoBuildMethod):
     #~ button_label = _("PDF")
     template_ext = '.pisa.html'  
     
-    def build(self,action,elem):
+    def build(self,ar,action,elem):
         tpl = self.get_template(action,elem) 
         filename = action.before_build(self,elem)
         if filename is None:
@@ -243,7 +259,7 @@ class SimpleBuildMethod(BuildMethod):
         #~ tpl = self.get_template_leaf(action,elem)
         #~ return self.templates_url + '/' + tpl
         
-    def build(self,action,elem):
+    def build(self,ar,action,elem):
         #~ if elem is None:
             #~ return
         from lino.utils.config import find_config_file
@@ -253,11 +269,11 @@ class SimpleBuildMethod(BuildMethod):
         tpl_leaf = self.get_template_leaf(action,elem)
         tplfile = find_config_file(tpl_leaf,self.get_group(elem))
         if not tplfile:
-            raise Exception("No file %s / %s" % (self.get_group(elem),tpl_leaf))
+            raise actions.Warning("No file %s / %s" % (self.get_group(elem),tpl_leaf))
         #~ tplfile = os.path.normpath(os.path.join(self.templates_dir,tpl_leaf))
-        return self.simple_build(elem,tplfile,target)
+        return self.simple_build(ar,elem,tplfile,target)
         
-    def simple_build(self,elem,tpl,target):
+    def simple_build(self,ar,elem,tpl,target):
         raise NotImplementedError
         
 class AppyBuildMethod(SimpleBuildMethod):
@@ -272,10 +288,10 @@ class AppyBuildMethod(SimpleBuildMethod):
     templates_name = 'appy' # subclasses use the same templates directory
     default_template = 'Default.odt'
     
-    def simple_build(self,elem,tpl,target):
+    def simple_build(self,ar,elem,tpl,target):
         #~ from lino.models import get_site_config
-        from appy.pod.renderer import Renderer
-        renderer = None
+        #~ from appy.pod.renderer import Renderer
+        #~ renderer = None
         context = dict(self=elem,
             dtos=babel.dtos,
             dtosl=babel.dtosl,
@@ -284,25 +300,24 @@ class AppyBuildMethod(SimpleBuildMethod):
             babelitem=babel.babelitem,
             tr=babel.babelitem,
             iif=iif,
+            mtos=decfmt,
             settings=settings,
+            ar=ar,
             #~ restify=restify,
             #~ site_config = get_site_config(),
-            site_config = settings.LINO.site_config,
+            site_config=settings.LINO.site_config,
             _ = _,
             #~ knowledge_text=fields.knowledge_text,
             )
         lang = str(elem.get_print_language(self))
         logger.info(u"appy.pod render %s -> %s (language=%r,params=%s",
             tpl,target,lang,settings.LINO.appy_params)
-        savelang = babel.get_language()
-        babel.set_language(lang)
-        #~ locale.setlocale(locale.LC_ALL,ls)
-        #~ Error: unsupported locale setting
-        renderer = Renderer(tpl, context, target,**settings.LINO.appy_params)
-        setup_renderer(renderer)
-        #~ renderer.context.update(restify=debug_restify)
-        renderer.run()
-        babel.set_language(savelang)
+        #~ savelang = babel.get_language()
+        #~ babel.set_language(lang)
+        def f():
+            Renderer(tpl, context, target,**settings.LINO.appy_params).run()
+        babel.run_with_language(lang,f)
+        #~ babel.set_language(savelang)
         return os.path.getmtime(target)
         
 
@@ -356,7 +371,7 @@ class LatexBuildMethod(BuildMethod):
     target_ext = '.pdf'
     template_ext = '.tex'  
     
-    def simple_build(self,elem,tpl,target):
+    def simple_build(self,ar,elem,tpl,target):
         context = dict(instance=elem)
         raise NotImplementedError
             
@@ -372,7 +387,7 @@ class RtfBuildMethod(SimpleBuildMethod):
     cache_name = 'userdocs'
     #~ cache_name = 'webdav'
     
-    def simple_build(self,elem,tpl,target):
+    def simple_build(self,ar,elem,tpl,target):
         context = dict(instance=elem)
         t = pyratemp.Template(filename=tpl)
         try:
@@ -414,45 +429,26 @@ def get_template_choices(elem,bmname):
     bm = bm_dict.get(bmname,None)
     if bm is None:
         raise Exception("%r : invalid print method name." % bmname)
-    from lino.utils.config import find_config_files
-    files = find_config_files('*' + bm.template_ext,bm.get_group(elem))
-    l = []
-    for name in files.keys():
-        # ignore babel variants: 
-        # e.g. ignore "foo_fr.odt" if "foo.odt" exists 
-        # but don't ignore "my_template.odt"
-        basename = name[:-len(bm.template_ext)]
-        chunks = basename.split('_')
-        if len(chunks) > 1:
-            basename = '_'.join(chunks[:-1])
-            if files.has_key(basename + bm.template_ext):
-                continue
-        l.append(name)
-    l.sort()
-    if not l:
-        logger.warning("get_template_choices() : no matches for (%r,%r)",'*' + bm.template_ext,bm.get_group(elem))
-    return l
+    return find_template_config_files(bm.template_ext,bm.get_group(elem))
+    
+    #~ files = find_config_files('*' + bm.template_ext,bm.get_group(elem))
+    #~ l = []
+    #~ for name in files.keys():
+        #~ # ignore babel variants: 
+        #~ # e.g. ignore "foo_fr.odt" if "foo.odt" exists 
+        #~ # but don't ignore "my_template.odt"
+        #~ basename = name[:-len(bm.template_ext)]
+        #~ chunks = basename.split('_')
+        #~ if len(chunks) > 1:
+            #~ basename = '_'.join(chunks[:-1])
+            #~ if files.has_key(basename + bm.template_ext):
+                #~ continue
+        #~ l.append(name)
+    #~ l.sort()
+    #~ if not l:
+        #~ logger.warning("get_template_choices() : no matches for (%r,%r)",'*' + bm.template_ext,bm.get_group(elem))
+    #~ return l
 
-def old_get_template_choices(group,bmname):
-    """
-    :param:bmname: the name of a build method.
-    """
-    pm = bm_dict.get(bmname,None)
-    if pm is None:
-        raise Exception("%r : invalid print method name." % bmname)
-    #~ glob_spec = os.path.join(pm.templates_dir,'*'+pm.template_ext)
-    top = os.path.join(pm.templates_dir,babel.DEFAULT_LANGUAGE,group)
-    l = []
-    for dirpath, dirs, files in os.walk(top):
-        for fn in files:
-            if fnmatch(fn,'*'+pm.template_ext):
-                if len(dirpath) > len(top):
-                    fn = os.path.join(dirpath[len(top)+1:],fn)
-                l.append(fn.decode(sys.getfilesystemencoding()))
-    if not l:
-        logger.warning("get_template_choices() : no matches for (%r,%r) in %s",group,bmname,top)
-    return l
-            
     
 def get_build_method(elem):
     bmname = elem.get_build_method()
@@ -476,16 +472,20 @@ def get_build_method(elem):
 
 #~ class PrintAction(actions.RedirectAction):
 class BasePrintAction(actions.RowAction):
-  
-    name = 'print'
+    """
+    Base class for all "Print" actions.
+    """
+    sort_index = 50
+    url_action_name = 'print'
     label = _('Print')
     
-    callable_from = (actions.GridEdit, actions.ShowDetailAction,
-        actions.ShowEmptyTable)
+    callable_from = (actions.GridEdit, 
+        actions.ShowDetailAction,
+        actions.ShowEmptyTable) # but not from InsertRow
     
-    def __init__(self,rpt,*args,**kw):
-        self.actor = rpt
-        actions.RowAction.__init__(self,*args,**kw)
+    #~ def __init__(self,rpt,*args,**kw):
+        #~ self.actor = rpt
+        #~ actions.RowAction.__init__(self,*args,**kw)
     
     def before_build(self,bm,elem):
         """Return the target filename if a document needs to be built,
@@ -522,11 +522,12 @@ class PrintAction(BasePrintAction):
     def get_print_templates(self,bm,elem):
         return elem.get_print_templates(bm,self)
         
-    def run_(self,request,ui,elem,**kw):
+    #~ def run_(self,request,ui,elem,**kw):
+    def run_(self,ar,elem,**kw):
         bm = get_build_method(elem)
         #~ if elem.must_build:
         if not elem.build_time:
-            t = bm.build(self,elem)
+            t = bm.build(ar,self,elem)
             if t is None:
                 raise Exception("%s : build() returned None?!")
                 #~ kw.update(message=_("%s printable has been built.") % elem)
@@ -539,32 +540,40 @@ class PrintAction(BasePrintAction):
                 kw.update(message=_("%s printable has been built.") % elem)
         else:
             kw.update(message=_("Reused %s printable from cache.") % elem)
-        url = bm.get_target_url(self,elem,ui)
+        url = bm.get_target_url(self,elem,ar.ui)
         if bm.use_webdav and settings.LINO.use_davlink:
-            kw.update(open_davlink_url=request.build_absolute_uri(url))
+            kw.update(open_davlink_url=ar.request.build_absolute_uri(url))
         else:
             kw.update(open_url=url)
         return kw
         #~ return rr.ui.success_response(open_url=target,**kw)
         
-    def run(self,rr,elem,**kw):
-        kw = self.run_(rr.request,rr.ui,elem,**kw)
-        return rr.ui.success_response(**kw)
+    def run(self,elem,ar,**kw):
+        #~ kw = self.run_(ar.request,rr.ui,elem,**kw)
+        kw = self.run_(ar,elem,**kw)
+        return ar.ui.success_response(**kw)
       
 class DirectPrintAction(BasePrintAction):
     """
-    A Print Action that uses a hard-coded template.
+    A Print Action that uses a hard-coded template and no cache.
     """
+    url_action_name = None
+    
     #~ def __init__(self,rpt,name,label,bmname,tplname):
-    def __init__(self,rpt,name=None,label=None,tplname='Default',build_method=None):
+    def __init__(self,label=None,tplname='Default',build_method=None,**kw):
         #~ if name is None: name = 'print'
         #~ if label is None: label = _("Print")
         #~ if tplname is None: tplname = 'Default'
-        BasePrintAction.__init__(self,rpt,name,label)
+        super(DirectPrintAction,self).__init__(label,**kw)
+        #~ BasePrintAction.__init__(self,rpt,name,label)
         #~ self.bm =  bm_dict.get(build_method or settings.LINO.preferred_build_method)
         self.build_method = build_method
         self.tplname = tplname
         
+    #~ def setup(self,actor,name):
+        #~ self.url_action_name = name
+        #~ super(DirectPrintAction,self).setup(actor,name)
+        
     def get_print_templates(self,bm,elem):
         #~ assert bm is self.bm
         if self.tplname:
@@ -573,21 +582,23 @@ class DirectPrintAction(BasePrintAction):
         #~ return super(DirectPrintAction,self).get_print_templates(bm,elem)
         
         
-    def run(self,rr,elem,**kw):
-        bm =  bm_dict.get(self.build_method or settings.LINO.site_config.default_build_method)
+    def run(self,elem,ar,**kw):
+        bm =  bm_dict.get(
+            self.build_method or 
+            settings.LINO.site_config.default_build_method)
         #~ if self.tplname:
             #~ if not self.tplname.endswith(bm.template_ext):
                 #~ raise Exception("Invalid template for build method %r" % bm.name)
-        bm.build(self,elem)
+        bm.build(ar,self,elem)
         #~ target = settings.MEDIA_URL + "/".join(bm.get_target_parts(self,elem))
         #~ return rr.ui.success_response(open_url=target,**kw)
-        url = bm.get_target_url(self,elem,rr.ui)
+        url = bm.get_target_url(self,elem,ar.ui)
         if bm.use_webdav and settings.LINO.use_davlink:
-            url = rr.request.build_absolute_uri(url)
+            url = ar.request.build_absolute_uri(url)
             kw.update(open_davlink_url=url)
         else:
             kw.update(open_url=url)
-        return rr.ui.success_response(**kw)
+        return ar.ui.success_response(**kw)
     
 #~ class EditTemplateAction(dd.RowAction):
     #~ name = 'tpledit'
@@ -602,21 +613,29 @@ class ClearCacheAction(actions.RowAction):
     """
     Defines the :guilabel:`Clear cache` button on a Printable record.
     """
-    name = 'clear'
+    sort_index = 11
+    url_action_name = 'clear'
     label = _('Clear cache')
     
     #~ def disabled_for(self,obj,request):
         #~ if not obj.build_time:
             #~ return True
+            
+    def get_action_permission(self,user,obj,state):
+        # obj may be None when Lino asks whether this action 
+        # should be visible in the UI
+        if obj is not None and not obj.build_time:
+            return False
+        return super(ClearCacheAction,self).get_action_permission(user,obj,state)
     
-    def run(self,rr,elem):
+    def run(self,elem,rr):
         t = elem.get_cache_mtime()
         if t is not None and t != elem.build_time:
             #~ logger.info("%r != %r", elem.get_cache_mtime(),elem.build_time)
             rr.confirm(
                 _("This will discard all changes in the generated file."),
                 _("Are you sure?"))
-            logger.info("Got confirmation to discard changes in %s", elem.get_cache_filename())
+            logger.info("Got confirmation to discard changes in %s", elem.get_target_name())
         #~ else:
             #~ logger.info("%r == %r : no confirmation", elem.get_cache_mtime(),elem.build_time)
           
@@ -624,8 +643,10 @@ class ClearCacheAction(actions.RowAction):
         elem.build_time = None
         elem.save()
         return rr.ui.success_response("%s printable cache has been cleared." % elem,refresh=True)
-
-class PrintableType(models.Model):
+        
+        
+    
+class PrintableType(Model):
     """
     Base class for models that specify the :attr:`TypedPrintable.type`.
     """
@@ -688,8 +709,12 @@ class Printable(object):
         return self._meta.app_label + '.' + self.__class__.__name__ + '-' + str(self.pk)
         
   
-class CachedPrintable(models.Model,Printable):
+class CachedPrintable(Duplicable,Printable):
+    """
+    Mixin for Models that generate a unique external file at a 
+    determined place when being printed.
     
+    """
     #~ must_build = models.BooleanField(_("must build"),default=True,editable=False)
     build_time = models.DateTimeField(_("build time"),null=True,editable=False)
     """
@@ -697,23 +722,33 @@ class CachedPrintable(models.Model,Printable):
     if no build hasn't been called yet.
     """
     
+    do_print = PrintAction()
+    do_clear_cache = ClearCacheAction()
+    
+    
     class Meta:
         abstract = True
         
-    @classmethod
-    def setup_report(cls,rpt):
-        #~ call_optional_super(CachedPrintable,cls,'setup_report',rpt)
-        rpt.add_action(PrintAction(rpt))
-        rpt.add_action(ClearCacheAction())
+    #~ @classmethod
+    #~ def setup_report(cls,rpt):
+        #~ rpt.add_action(PrintAction())
+        #~ rpt.add_action(ClearCacheAction())
         
-    def get_permission(self,action,user):
-        if isinstance(action,ClearCacheAction):
-            if not self.build_time:
-                return False
-        return True
+    #~ def get_row_permission(self,user,state,action):
+        #~ """
+        #~ Cached printables may not be edited after they have been printed.
+        #~ """
+        #~ if self.build_time and not action.readonly:
+            #~ return False
+        #~ return super(CachedPrintable,self).get_row_permission(user,state,action)
       
+    def print_from_posting(self,posting,ar):
+        return self.do_print.run(self,ar)
+        
+    def on_duplicate(self,ar,master):
+        super(CachedPrintable,self).on_duplicate(ar,master)
+        self.build_time = None
         
-
     def get_print_templates(self,bm,action):
         """Return a list of filenames of templates for the specified build method.
         Returning an empty list means that this item is not printable. 
@@ -734,14 +769,15 @@ class CachedPrintable(models.Model,Printable):
         #~ return settings.LINO.preferred_build_method 
         #~ return 'pisa'
         
-    def get_cache_filename(self):
-        # TODO: too stupid that we must instantate an Action here...
-        a = PrintAction(self.__class__._lino_model_report)
-        bm = get_build_method(self)
-        return bm.get_target_name(a,self)
+    def get_target_name(self):
+        if self.build_time:
+            return get_build_method(self).get_target_name(self.do_print,self)
+        
+    def get_target_url(self,ui):
+        return get_build_method(self).get_target_url(self.do_print,self,ui)
         
     def get_cache_mtime(self):
-        filename = self.get_cache_filename()
+        filename = self.get_target_name()
         if not filename: return None
         try:
             t = os.path.getmtime(filename)
@@ -753,6 +789,9 @@ class CachedPrintable(models.Model,Printable):
 
 class TypedPrintable(CachedPrintable):
     """
+    A :class:`CachedPrintable` that uses a "Type" for deciding which template 
+    to use on a given instance. 
+    
     A TypedPrintable model must define itself a field `type` which is a ForeignKey 
     to a Model that implements :class:`PrintableType`.
     
@@ -792,139 +831,13 @@ class TypedPrintable(CachedPrintable):
             return super(TypedPrintable,self).get_print_templates(bm,action)
         tplname = ptype.template or bm.default_template
         if not tplname.endswith(bm.template_ext):
-            raise Exception(
+            raise actions.Warning(
               "Invalid template %r configured for %s %r (expected filename ending with %r)" %
               (tplname,ptype.__class__.__name__,unicode(ptype),bm.template_ext))
         return [ tplname ]
         #~ return [ ptype.get_templates_group() + '/' + ptype.template ]
         
-    def get_print_language(self,bm):
-        return self.language
+    #~ def get_print_language(self,bm):
+        #~ return self.language
 
 
-
-  
-import cgi
-
-
-class Listing(CachedPrintable):
-    """
-    Abstract base class for all Listings. 
-    A Listing is a printable report that requires some parameters given by the user.
-    Subclasses must implement the :meth:`body` method.
-    Each time a user asks to print a Listing, Lino will create a new record.
-    """
-    #~ template_name = 'Listing.odt'
-    template_name = None
-    build_method = None
-    
-    class Meta:
-        abstract = True
-    
-    date = models.DateField(
-        blank=True,null=True,
-        verbose_name=_("Date"))
-    #~ header_html = dd.RichTextField(_("Header"),editable=False)
-    #~ footer_html = dd.RichTextField(_("Footer"),editable=False)
-    body_html = fields.RichTextField(_("Body"),editable=False)
-        
-    #~ title = models.CharField(max_length=200,
-      #~ verbose_name=_("Title"),
-      #~ blank=True)
-    #~ """
-    #~ The title of the listing.
-    #~ """
-    
-    @classmethod
-    def unused_setup_report(model,rpt):
-        u"""
-        """
-        # to not call call_optional_super(Listing,self,'setup_report',rpt)
-        #~ rpt.get_action('listing').label = model.__name__
-        rpt.add_action(DirectPrintAction(rpt,'print',_("Print"),
-          model.template_name,model.build_method))
-        #~ rpt.add_action(InititateListing('listing',_("Print"),'listing.odt'))
-        
-    def __unicode__(self):
-        return force_unicode(self.title)
-        #~ return self.get_title()
-        
-    #~ def get_templates_group(self):
-        #~ return ''
-        
-    def get_title(self):
-        return self._meta.verbose_name # "Untitled Listing"
-    title = property(get_title)
-        
-    #~ def header(self):
-        #~ return '<p align="center"><b>%s</b></p>' % cgi.escape(self.title)
-        
-    #~ def footer(self):
-        #~ html = '<td align="left">%s</td>' % 'left footer'
-        #~ html += '<td align="right">Page X of Y</td>'
-        #~ html = '<table width="100%%"><tr>%s</tr></table>' % html
-        #~ return html
-        
-    def body(self):
-        """
-        To be implemented by subclasses.
-        Build the XHTML content of the listing to be printed.
-        """
-        raise NotImplementedError
-        
-    #~ def preview(self,request):
-        #~ return self.header() + self.body() + self.footer()
-    #~ preview.return_type = fields.HtmlBox(_("Preview"))
-    def save(self,*args,**kw):
-        #~ self.header_html = self.header()
-        #~ self.footer_html = self.footer()
-        if not self.body_html:
-            self.body_html = self.body()
-        super(Listing,self).save(*args,**kw)
-    
-    def get_preview(self,request):
-        return self.body_html
-        #~ return self.header_html + self.body_html + self.footer_html
-    preview = fields.VirtualField(fields.HtmlBox(_("Preview")),get_preview)
-    
-    
-class InitiateListing(actions.InsertRow):
-    """
-    This is the (otherwise invisible) action which is used 
-    for the main menu entry of a :class:`Listing`.
-    """
-    callable_from = tuple()
-    name = 'listing'
-    #~ label = _("Initiate")
-    key = None
-    
-    def get_action_title(self,rh):
-        return _(u"Initiate Listing «%s»") % self.actor.model._meta.verbose_name
-  
-    def get_button_label(self):
-        return self.actor.model._meta.verbose_name
-        
-class Listings(dd.Table):
-    model = Listing
-    
-    @classmethod
-    def init_label(self):
-        return _(u"Listings «%s»") % self.model._meta.verbose_name
-        
-    @classmethod
-    def setup_actions(self):
-        #~ print 'lino.mixins.printable.Listings.setup_actions : ', self.model
-        alist = []
-        #~ if len(self.detail_layouts) > 0:
-        if True:
-            self.detail_action = actions.ShowDetailAction(self)
-            alist.append(self.detail_action)
-            alist.append(actions.SubmitDetail())
-            alist.append(InitiateListing(self,label=self.model._meta.verbose_name)) # replaces InsertRow
-            alist.append(actions.SubmitInsert())
-            self.default_action = actions.GridEdit(self)
-            #~ alist.append(self.default_action)
-        alist.append(actions.DeleteSelected())
-        self.set_actions(alist)
-        
-    
\ No newline at end of file
