@@ -38,14 +38,44 @@ from lino import dd
 #~ from lino import commands
 from lino.mixins import printable
 from lino.utils import babel
-from lino.utils import perms
 #~ from lino import choices_method, simple_choices_method
-from lino.tools import obj2str, sorted_models_list
-from lino.tools import resolve_field
-from lino.utils.choosers import chooser
+from lino.core.modeltools import obj2str, sorted_models_list
+from lino.core.modeltools import resolve_field
+from lino.utils.choosers import chooser, get_for_field
+#~ from lino.modlib.users.models import UserLevels
+from lino.utils.restify import restify
 from lino.core import actions
 
-class SiteConfig(models.Model):
+
+
+
+
+
+#~ class BuildLinoJS(dd.RowAction):
+class BuildSiteCache(dd.RowAction):
+    """
+    Rebuild the site cache.
+    This action is available on :class:`About`.
+    """
+    label = _("Rebuild site cache")
+    url_action_name = "buildjs"
+    def run(self,elem,rr):
+        rr.confirm(_("Are you sure?"))
+        #~ rr.confirm(_("Are you really sure?"))
+        rr.ui.build_site_cache(True)
+        return rr.ui.success_response(
+            """\
+Seems that it worked. Refresh your browser. 
+<br>
+Note that other users might experience side effects because 
+of the unexpected .js update, but there are no known problems so far.
+Please report any anomalies.""",
+            alert=_("Success"))
+
+    
+
+
+class SiteConfig(dd.Model):
     """
     This model should have exactly one instance, 
     used to store persistent global site parameters.
@@ -57,15 +87,14 @@ class SiteConfig(models.Model):
       default='appyodt',
       choices=printable.build_method_choices(),blank=True)
         
-    #~ def save(self,*args,**kw):
-        #~ settings.LINO.configure(self)
-        #~ r = super(SiteConfig,self).save(*args,**kw)
-        #~ return r
+    def save(self,*args,**kw):
+        super(SiteConfig,self).save(*args,**kw)
+        settings.LINO.on_site_config_saved(self)
    
     def __unicode__(self):
         return force_unicode(_("Site Parameters"))
 
-#~ class SiteConfigDetail(dd.DetailLayout):
+#~ class SiteConfigDetail(dd.FormLayout):
     #~ about = """
     #~ versions:40x5 startup_time:30
     #~ lino.ModelsBySite:70x10
@@ -80,75 +109,26 @@ class SiteConfig(models.Model):
         #~ lh.about.label = _("About")
     
 class SiteConfigs(dd.Table):
+    """
+    The table used to present the :class:`SiteConfig` row in a Detail form.
+    See also :meth:`lino.Lino.get_site_config`.
+    Deserves more documentation.
+    """
+    default_action = actions.ShowDetailAction()
     model = SiteConfig
+    required = dict(user_level='manager')
     #~ default_action_class = dd.OpenDetailAction
     #~ has_navigator = False
     hide_top_toolbar = True
     #~ can_delete = perms.never
     detail_template = """
     default_build_method
-    lino.ModelsBySite
+    # lino.ModelsBySite
     """
-    #~ detail_layout = SiteConfigDetail()
-    #~ editable = True
-    
-    #~ window_size = (700,400)
-    #~ detail_layout = AboutDetail()
-    #~ detail_template = """
-    #~ """
-    
-    @classmethod
-    def get_permission(self,action,user,obj):
-        if not user.is_superuser:
-            return action.readonly
-        #~ if isinstance(action,actions.DeleteSelected):
-            #~ return False
-        return True
         
-    @classmethod
-    def setup_actions(self):
-        super(SiteConfigs,self).setup_actions()
-        self.add_action(BuildLinoJS())
-        #~ self.remove_action(actions.DeleteSelected
-   
-    #~ @dd.constant(_("Versions"))
-    #~ def versions(cls,ui):
-        #~ return lino.welcome_html(ui)
-        
-    #~ @dd.displayfield(_("Versions"))
-    #~ def versions(self,obj,ar):
-        #~ return lino.welcome_html(ar.ui)
-        
-    #~ @dd.constantfield(_("Versions"))
-    #~ def versions(cls,self,req):
-        #~ return lino.welcome_html()
-        
-    #~ @dd.virtualfield(models.DateTimeField(_("Server up since")))
-    #~ def startup_time(cls,self,req):
-        #~ return settings.LINO.startup_time
+    do_build = BuildSiteCache()
     
 
-    
-    
-def get_site_config():
-    try:
-        return SiteConfig.objects.get(pk=1)
-    #~ except SiteConfig.DoesNotExist:
-    except Exception,e:
-        kw = dict(pk=1)
-        kw.update(settings.LINO.site_config_defaults)
-        logger.debug("Creating SiteConfig record (%s)",e)
-        sc = SiteConfig(**kw)
-        #~ do NOT save the instance here
-        #~ sc.save()
-        return sc
-
-def update_site_config(**kw):
-    sc = get_site_config()
-    for k,v in kw.items():
-        setattr(sc,k,v)
-    sc.save()
-
 if settings.LINO.is_installed('contenttypes'):
 
   class ContentTypes(dd.Table):
@@ -168,11 +148,12 @@ if settings.LINO.is_installed('contenttypes'):
           def add(cl):
               for b in cl.__bases__:
                   add(b)
-              if issubclass(cl,models.Model) and cl is not models.Model and cl._meta.managed: # :
+              if issubclass(cl,dd.Model) and cl is not dd.Model and cl._meta.managed: # :
                   if getattr(cl,'_meta',False) and not cl._meta.abstract:
-                      logger.info("20120205 adding(%r)",cl)
+                      #~ logger.info("20120205 adding(%r)",cl)
                       ct = contenttypes.ContentType.objects.get_for_model(cl)
-                      chunks.append(ar.ui.ext_renderer.href_to(ct,unicode(cl._meta.verbose_name)))
+                      #~ chunks.append(ar.ui.ext_renderer.href_to(ct,unicode(cl._meta.verbose_name)))
+                      chunks.append(ar.href_to(ct,unicode(cl._meta.verbose_name)))
           if obj is not None:
               #~ add(obj.model_class())
               for b in obj.model_class().__bases__:
@@ -181,7 +162,7 @@ if settings.LINO.is_installed('contenttypes'):
       
       
       
-  class HelpText(models.Model):
+  class HelpText(dd.Model):
       
       class Meta:
           verbose_name = _("Help Text")
@@ -189,8 +170,7 @@ if settings.LINO.is_installed('contenttypes'):
           
       content_type = models.ForeignKey(contenttypes.ContentType,
           verbose_name=_("Model"))
-      field = models.CharField(_("Field"),
-          max_length=200)
+      field = models.CharField(_("Field"),max_length=200)
 
       help_text = dd.RichTextField(_("HelpText"),
           blank=True,null=True,format='plain')
@@ -202,7 +182,8 @@ if settings.LINO.is_installed('contenttypes'):
       def field_choices(cls,content_type):
           l = []
           if content_type is not None:
-              meta = content_type.model_class()._meta
+              model = content_type.model_class()
+              meta = model._meta
               #~ for f in meta.fields: yield f.name
               #~ for f in meta.many_to_many: yield f.name
               #~ for f in meta.virtual_fields: yield f.name
@@ -211,6 +192,9 @@ if settings.LINO.is_installed('contenttypes'):
                       l.append(f.name)
               for f in meta.many_to_many: l.append(f.name)
               for f in meta.virtual_fields: l.append(f.name)
+              for a in model._lino_default_table.get_actions():
+                  l.append(a.name)
+              l.sort()
           return l
           
       #~ def get_field_display(cls,fld):
@@ -218,7 +202,18 @@ if settings.LINO.is_installed('contenttypes'):
 
       @dd.virtualfield(models.CharField(_("Verbose name"),max_length=200))
       def verbose_name(self,request):
-          return resolve_field(unicode(self)).verbose_name
+          #~ return unicode(self)
+          m = dd.resolve_model(self.content_type.app_label + '.' + self.content_type.name)
+          de = m._lino_default_table.get_data_elem(self.field)
+          if isinstance(de,models.Field):
+              #~ return unicode(de.verbose_name)
+              return "%s (%s)" % (unicode(de.verbose_name), unicode(_("database field")))
+          if isinstance(de,dd.VirtualField):
+              return unicode(de.return_type.verbose_name)
+          if isinstance(de,actions.Action):
+              return unicode(de.label)
+          return str(de)
+          #~ return unicode(resolve_field(unicode(self)).verbose_name)
               
               
               
@@ -228,41 +223,9 @@ if settings.LINO.is_installed('contenttypes'):
       
   class HelpTextsByModel(HelpTexts):
       master_key = 'content_type'
+      
 
 
-if False:
-  
-    class DataControlListing(mixins.Listing):
-        """Performs a "soft integrity test" on the database. 
-        Prints 
-        """
-        class Meta:
-            verbose_name = _("Data Control Listing") 
-        
-        def body(self):
-            items = []
-            for model in sorted_models_list():
-                m = getattr(model,'data_control',None)
-                if m is not None:
-                    for i in model.objects.all():
-                        msgs = i.data_control()
-                        if msgs:
-                            if len(msgs) == 1:
-                                items.append("<b>%s</b> : %s" % (unicode(i),msgs[0]))
-                            else:
-                                items.append("<b>%s</b> : %s" % (
-                                  unicode(i),
-                                  "\n".join(
-                                    ["<br>(%d) %s" % (x[0]+1,x[1])
-                                      for x in enumerate(msgs)])))
-            #~ html = "<ol>"
-            #~ html += "\n".join(["<li>%s</li>" % ln for ln in items])
-            #~ html += "</ol>"
-            html = "\n".join(["<p>%s</p>" % ln for ln in items])
-            html = '<div class="htmlText">%s</div>' % html
-            return html
-        
-
 #~ if settings.LINO.is_installed('users'):
 if settings.LINO.user_model:
 
@@ -297,62 +260,36 @@ if settings.LINO.user_model:
         
 
 
-class BuildLinoJS(dd.RowAction):
-    """
-    Rebuild the :xfile`lino.js` files.
-    This action is available on :class:`About`.
-    """
-    label = _("Build lino*.js files")
-    name = "buildjs"
-    def run(self,rr,elem):
-        rr.confirm(_("Are you sure?"))
-        #~ rr.confirm(_("Are you really sure?"))
-        rr.ui.build_lino_js(True)
-        return rr.ui.success_response(
-            """\
-Seems that it worked. Refresh your browser. 
-<br>
-Note that other users might experience side effects because 
-of the unexpected .js update, but there are no known problems so far.
-Please report any anomalies.""",
-            alert=_("Success"))
 
+#~ class ModelsBySite(dd.VirtualTable):
+    #~ label = _("Models")
+    #~ column_names = "app name docstring rows"
+    #~ master = SiteConfig
     
-
-class ModelsBySite(dd.VirtualTable):
-    label = _("Models")
-    #~ column_defaults = dict(width=8)
-    #~ column_names = "app name verbose_name docstring rows"
-    column_names = "app name docstring rows"
-    master = SiteConfig
-    
-    slave_grid_format = 'html'    
+    #~ slave_grid_format = 'html'    
   
-    @classmethod
-    def get_data_rows(self,ar):
-        for model in models.get_models():
-            yield model
+    #~ @classmethod
+    #~ def get_data_rows(self,ar):
+        #~ for model in models.get_models():
+            #~ yield model
                 
-    @dd.displayfield(_("app_label"))
-    def app(self,obj,ar):
-        return obj._meta.app_label
+    #~ @dd.displayfield(_("app_label"))
+    #~ def app(self,obj,ar):
+        #~ return obj._meta.app_label
         
-    @dd.displayfield(_("name"))
-    def name(self,obj,ar):
-        return obj.__name__
+    #~ @dd.displayfield(_("name"))
+    #~ def name(self,obj,ar):
+        #~ return obj.__name__
         
-    #~ @dd.displayfield(_("verbose name"))
-    #~ def vebose_name(self,obj,ar):
-        #~ return unicode(obj._meta.vebose_name)
+    #~ @dd.displayfield(_("docstring"))
+    #~ def docstring(self,obj,ar):
+        #~ return obj.__doc__
         
-    @dd.displayfield(_("docstring"))
-    def docstring(self,obj,ar):
-        return obj.__doc__
+    #~ @dd.requestfield(_("Rows"))
+    #~ def rows(self,obj,ar):
+        #~ return obj._lino_default_table.request(ar.ui,
+          #~ user=ar.get_user(),renderer=ar.renderer)
         
-    @dd.requestfield(_("Rows"))
-    def rows(self,obj,ar):
-        return obj._lino_model_report.request(ar.ui,
-          user=ar.get_user(),renderer=ar.renderer)
         
 class Models(dd.VirtualTable):
     label = _("Models")
@@ -360,6 +297,10 @@ class Models(dd.VirtualTable):
     #~ column_names = "app name verbose_name docstring rows"
     column_names = "app name docstring rows"
     #~ master = SiteConfig
+    detail_template = """
+    app name docstring rows
+    lino.FieldsByModel
+    """
     
     slave_grid_format = 'html'    
   
@@ -382,13 +323,41 @@ class Models(dd.VirtualTable):
         
     @dd.displayfield(_("docstring"))
     def docstring(self,obj,ar):
-        return obj.__doc__
+        #~ return obj.__doc__
+        return restify(unicode(obj.__doc__))
         
     @dd.requestfield(_("Rows"))
     def rows(self,obj,ar):
-        return obj._lino_model_report.request(ar.ui,
+        return obj._lino_default_table.request(ar.ui,
           user=ar.get_user(),renderer=ar.renderer)
+
+
+class FieldsByModel(dd.VirtualTable):
+    label = _("Fields")
+    #~ master_key = "model"
+    master = Models
+    column_names = "name verbose_name help_text"
+    
+    @classmethod
+    def get_data_rows(self,ar):
+        model = ar.master_instance
+        if model:
+            for (fld,remote) in model._meta.get_fields_with_model():
+                yield fld
+                
+    @dd.displayfield(_("name"))
+    def name(self,fld,ar):
+        return fld.name
         
+    @dd.displayfield(_("verbose name"))
+    def verbose_name(self,fld,ar):
+        return unicode(fld.vebose_name)
+        
+    @dd.displayfield(_("help text"))
+    def help_text(self,obj,ar):
+        #~ return obj.__doc__
+        return restify(unicode(obj.help_text))
+
 
 import inspect
 import types
@@ -402,7 +371,13 @@ class Inspected(object):
         self.value = value
 
 class Inspector(dd.VirtualTable):
+    """
+    Shows a simplistic "inspector" which once helped me for debugging.
+    Needs more work to become seriously useful...
+    
+    """
     label = _("Inspector")
+    required = dict(user_level='admin')
     column_names = "i_name i_type i_value"
     parameters = dict(
       inspected=models.CharField(_("Inspected object"),max_length=100,blank=True),
@@ -484,7 +459,8 @@ class Inspector(dd.VirtualTable):
             pv.update(inspected=ar.param_values.inspected+obj.prefix+obj.name)
         else:
             pv.update(inspected=obj.name)
-        newreq = ar.report.request(ar.ui,user=ar.user,renderer=ar.renderer,param_values=pv)
+        #~ newreq = ar.spawn(ar.ui,user=ar.user,renderer=ar.renderer,param_values=pv)
+        newreq = ar.spawn(param_values=pv)
         return ar.renderer.href_to_request(newreq,obj.name)
         #~ return obj.name
         
@@ -497,34 +473,40 @@ class Inspector(dd.VirtualTable):
         return cgi.escape(str(type(obj.value)))
         
 
-#~ class AboutDetail(dd.DetailLayout):
-  
-    #~ main = """
-    #~ versions:40x5 startup_time:30
-    #~ lino.Models:70x10
-    #~ """
-
-
-class About(dd.EmptyTable):
+class AboutDetail(dd.FormLayout):
     """
-    A modal window displaying information about this Lino server.
+    The Detail Layout for :class:`About`
+    """
+    window_size = (60,30)
+    main = """
+    versions:40x5 startup_time:30
+    lino.Models:70x10
+    """
+
+
+class About(mixins.EmptyTable):
+    """
+    A modal window displaying information about this server.
     """
     label = _("About") 
     #~ hide_window_title = True
     hide_top_toolbar = True
-    window_size = (700,400)
-    #~ detail_layout = AboutDetail()
-    detail_template = """
-    versions:40x5 startup_time:30
-    lino.Models:70x10
-    """
+    #~ window_size = (700,400)
+    #~ detail_layout = AboutDetail(window_size = (700,400))
+    detail_layout = AboutDetail()
+    #~ detail_template = """
+    #~ versions:40x5 startup_time:30
+    #~ lino.Models:70x10
+    #~ """
     
     #~ versions = dd.Constant(lino.welcome_html())
     
-    @classmethod
-    def setup_actions(self):
-        super(About,self).setup_actions()
-        self.add_action(BuildLinoJS())
+    do_build = BuildSiteCache()
+    
+    #~ @classmethod
+    #~ def setup_actions(self):
+        #~ super(About,self).setup_actions()
+        #~ self.add_action(BuildSiteCache())
    
     #~ @dd.constant(_("Versions"))
     @dd.constant()
@@ -545,11 +527,12 @@ class About(dd.EmptyTable):
     
 
 
-class Home(dd.EmptyTable):
+class Home(mixins.EmptyTable):
     """
     This is the "home page" or "welcome screen", the window to be displayed 
     when no other window is opened.
     """
+    #~ debug_actions = True
     label = _("Home") 
     hide_window_title = True
     hide_top_toolbar = True
@@ -559,10 +542,10 @@ class Home(dd.EmptyTable):
     welcome
     """
     
-    @classmethod
-    def setup_actions(self):
-        "Overrides the default method. Home page needs no print method."
-        pass
+    #~ @classmethod
+    #~ def setup_actions(self):
+        #~ "Overrides the default method. Home page needs no print method."
+        #~ pass
         
     #~ @dd.virtualfield(dd.HtmlBox())
     #~ def tasks_summary(cls,self,req):
@@ -572,8 +555,9 @@ class Home(dd.EmptyTable):
     def quick_links(cls,self,req):
         quicklinks = settings.LINO.get_quicklinks(self,req.get_user())
         if quicklinks.items:
+            #~ assert mi.params is None
             return 'Quick Links: ' + ' '.join(
-              [req.ui.ext_renderer.action_href_js(mi.action,mi.params) 
+              [req.ui.ext_renderer.action_href_js(mi.action) 
                 for mi in quicklinks.items]
               )
       
@@ -603,35 +587,31 @@ class Home(dd.EmptyTable):
 
 
 
-#~ def add_site_menu(site):
-    #~ m = site.add_menu("site",_("~Site"))
-    #~ m.add_instance_action(site.config,
-        #~ label=_('Global Site Parameters'),
-        #~ can_view=perms.is_staff)
-    #~ return m
-
 
 def setup_main_menu(site,ui,user,m): pass
 
 def setup_my_menu(site,ui,user,m): pass
   
 def setup_config_menu(site,ui,user,m):
-    config_etc      = m.add_menu("etc",_("System"))
-    #~ config_etc.add_action('links.LinkTypes')
+    m = m.add_menu("etc",_("System"))
+    #~ m.add_action('links.LinkTypes')
     if site.user_model:
-        config_etc.add_action(site.user_model)
-        config_etc.add_action(site.modules.lino.TextFieldTemplates)
-    #~ config_etc.add_action(site.modules.users.Users)
+        m.add_action(site.user_model)
+        m.add_action(site.modules.lino.TextFieldTemplates)
+    #~ m.add_action(site.modules.users.Users)
     if site.is_installed('contenttypes'):
-    #~ if site.use_contenttypes:
-        config_etc.add_action(site.modules.lino.ContentTypes)
-        config_etc.add_action(site.modules.lino.HelpTexts)
+        m.add_action(site.modules.lino.ContentTypes)
+        m.add_action(site.modules.lino.HelpTexts)
+        #~ m.add_action(site.modules.lino.Workflows)
     #~ if site.use_tinymce:
-    config_etc.add_instance_action(site.site_config)
+    m.add_instance_action(site.site_config)
         
   
-def setup_explorer_menu(site,ui,user,m): pass
+def setup_explorer_menu(site,ui,user,m):
+    m.add_action(site.modules.users.Authorities)
+  
   
 def setup_site_menu(site,ui,user,m): 
     m.add_action(site.modules.lino.About)
     m.add_action(site.modules.lino.Inspector)
+
