@@ -53,10 +53,9 @@ from lino.modlib.postings import models as postings
 
 
 from lino.modlib.cal.utils import \
-    Weekday, DurationUnits, setkw, dt2kw, EventState, GuestState, TaskState, AccessClasses
-#~ from lino.modlib.cal.utils import EventStatus, \
-    #~ TaskStatus, DurationUnit, Priority, AccessClass, \
-    #~ GuestStatus, setkw, dt2kw
+    Weekday, DurationUnits, setkw, dt2kw, \
+    EventState, GuestState, TaskState, AccessClasses, \
+    CalendarAction
 
 from lino.utils.babel import dtosl
 
@@ -165,8 +164,8 @@ class Calendar(mixins.PrintableType,outbox.MailableType,babel.BabelNamed):
               password=self.password)
         return ''
                     
-    def __unicode__(self):
-        return self.name
+    #~ def __unicode__(self):
+        #~ return self.name
         
     #~ def color(self,request):
         #~ return settings.LINO.get_calendar_color(self,request)
@@ -253,44 +252,44 @@ class SubscriptionsByUser(Subscriptions):
 #~ class MySubscriptions(Subscriptions,mixins.ByUser):
     #~ pass
 
+if settings.LINO.user_model:
 
-
-class Membership(mixins.UserAuthored):
-    """
-    A Membership is when a User decides that subscribes to somebody else's Calendar.
-    
-    :user: points to the author (recipient) of this membership
-    :watched_user: points to the watched user
-    
-    """
-    
-    class Meta:
-        verbose_name = _("Membership")
-        verbose_name_plural = _("Memberships")
+    class Membership(mixins.UserAuthored):
+        """
+        A Membership is when a User decides that subscribes to somebody else's Calendar.
         
-    #~ quick_search_fields = ('user__username','user__first_name','user__last_name')
-    
-    watched_user = models.ForeignKey(settings.LINO.user_model,
-        help_text=_("""\
-The user whose calendar events you want to see in team view."""))
-
-
-
-    @dd.chooser()
-    def watched_user_choices(cls,user):
-        return settings.LINO.user_model.objects.exclude(
-            profile=dd.UserProfiles.blank_item).exclude(id=user.id)
-    
+        :user: points to the author (recipient) of this membership
+        :watched_user: points to the watched user
         
-class Memberships(dd.Table):
-    required = dict(user_groups='office',user_level='manager')
-    model = Membership
+        """
+        
+        class Meta:
+            verbose_name = _("Membership")
+            verbose_name_plural = _("Memberships")
+            
+        #~ quick_search_fields = ('user__username','user__first_name','user__last_name')
+        
+        watched_user = models.ForeignKey(settings.LINO.user_model,
+            help_text=_("""\
+    The user whose calendar events you want to see in team view."""))
 
 
-class MembershipsByUser(Memberships):
-    required = dict(user_groups='office')
-    master_key = 'user'
-    label = _("Team Members")
+
+        @dd.chooser()
+        def watched_user_choices(cls,user):
+            return settings.LINO.user_model.objects.exclude(
+                profile=dd.UserProfiles.blank_item).exclude(id=user.id)
+        
+            
+    class Memberships(dd.Table):
+        required = dict(user_groups='office',user_level='manager')
+        model = Membership
+
+
+    class MembershipsByUser(Memberships):
+        required = dict(user_groups='office')
+        master_key = 'user'
+        label = _("Team Members")
 
 
 
@@ -331,6 +330,7 @@ class Priority(babel.BabelNamed):
         verbose_name = _("Priority")
         verbose_name_plural = _('Priorities')
     ref = models.CharField(max_length='1')
+
 class Priorities(dd.Table):
     required = dict(user_groups='office')
     model = Priority
@@ -398,7 +398,11 @@ class EventGenerator(mixins.UserAuthored):
         
     def save(self,*args,**kw):
         super(EventGenerator,self).save(*args,**kw)
-        self.update_reminders()
+        babel.run_with_language(self.user.language,self.update_reminders)
+        #~ lang = babel.get_language()
+        #~ babel.set_language(self.user.language)
+        #~ self.update_reminders()
+        #~ babel.set_language(lang)
   
     def update_cal_rset(self):
         return self.exam_policy
@@ -855,8 +859,8 @@ Whether this is private, public or between.""")) # iCal:CLASS
     def is_user_modified(self):
         return self.state
         
-    def on_user_change(self,request):
-        raise NotImplementedError
+    #~ def on_user_change(self,request):
+        #~ raise NotImplementedError
         #~ self.user_modified = True
         
     #~ def summary_row(self,ui,rr,**kw):
@@ -870,7 +874,7 @@ Whether this is private, public or between.""")) # iCal:CLASS
         #~ logger.info("20120217 Component.summary_row() %s", self)
         #~ if self.owner and not self.auto_type:
         #~ html = ui.ext_renderer.href_to(self)
-        html = ar.renderer.href_to(self)
+        html = ar.href_to(self)
         if self.start_time:
             #~ html += _(" at ") + unicode(self.start_time)
             html += _(" at ") + self.start_time.strftime(settings.LINO.time_format_strftime)
@@ -1001,13 +1005,12 @@ Indicates that this Event shouldn't prevent other Events at the same time."""))
             #~ self.state = EventState.obsolete
         super(Event,self).save(*args,**kw)
         if self.calendar and self.calendar.invite_team_members:
-            if True or self.access_class == AccessClasses.public:
-                if not self.state in (EventState.blank_item, EventState.draft):
-                    if self.guest_set.all().count() == 0:
-                        #~ print 20120711
-                        for obj in Membership.objects.filter(user=self.user):
-                            if obj.watched_user.partner:
-                                Guest(event=self,partner=obj.watched_user.partner).save()
+            if not self.state in (EventState.blank_item, EventState.draft):
+                if self.guest_set.all().count() == 0:
+                    #~ print 20120711
+                    for obj in Membership.objects.filter(user=self.user):
+                        if obj.watched_user.partner:
+                            Guest(event=self,partner=obj.watched_user.partner).save()
         
     def after_state_change(self,ar,kw,old,new):
         """
@@ -1027,10 +1030,25 @@ Indicates that this Event shouldn't prevent other Events at the same time."""))
             #~ kw['message'] += '\n(' + _("Event remains *%s*.") % self.state + ')'
                 
             
-    def on_user_change(self,request):
+    def before_ui_save(self,ar,**kw):
+        """
+        Mark the event as "user modified" by setting a default state.
+        This is important because EventGenerators may not modify any user-modified Events.
+        """
         if not self.state:
-            self.state = EventState.draft
-        #~ self.user_modified = True
+            if ar.request.subst_user:
+                self.state = EventState.suggested
+                #~ self.state = EventState.reserved
+            else:
+                self.state = EventState.draft
+        return super(Event,self).before_ui_save(ar,**kw)
+        
+    #~ def on_user_change(self,request):
+        #~ if not self.state:
+            #~ if request.subst_user:
+                #~ self.state = EventState.reserved
+            #~ else:
+                #~ self.state = EventState.draft
         
     def on_create(self,ar):
         self.start_date = datetime.date.today()
@@ -1229,16 +1247,25 @@ if settings.LINO.project_model:
 if settings.LINO.user_model:    
   
     class MyEvents(Events,mixins.ByUser):
-        required = dict(user_groups='office')
         help_text = _("Table of all my calendar events.")
+        required = dict(user_groups='office')
         column_names = 'start_date start_time calendar project summary state workflow_buttons *'
-        #~ model = 'cal.Event'
-        #~ label = _("My Events")
-        #~ column_names = 'start_date start_time summary state *'
+        
+    #~ class EventsReserved(Events):
+        #~ help_text = _("Table of all reserved events.")
+        #~ label = _("Reserved Events")
+        #~ required = dict(user_groups='office',user_level='manager')
+        #~ column_names = 'start_date start_time user project summary workflow_buttons *'
+        #~ known_values = dict(state=EventState.reserved)
+        
+    #~ class MyEventsReserved(EventsReserved,MyEvents):
+        #~ help_text = _("Table of my reserved events (to become scheduled).")
+        #~ required = dict(user_groups='office')
+        #~ column_names = 'start_date start_time project summary workflow_buttons *'
+        #~ label = _("My reserved Events")
+        
         
     class EventsSuggested(Events):
-        """
-        """
         help_text = _("Table of all suggested events.")
         label = _("Suggested Events")
         required = dict(user_groups='office',user_level='manager')
@@ -1252,8 +1279,6 @@ if settings.LINO.user_model:
         label = _("My suggested Events")
         
     class EventsNotified(Events):
-        """
-        """
         help_text = _("Table of all notified events (waiting to become scheduled).")
         label = _("Notified Events")
         required = dict(user_groups='office',user_level='manager')
@@ -1409,9 +1434,14 @@ class Task(Component):
         #~ return ar.success_response(refresh=True)
     
 
-    def on_user_change(self,request):
+    def before_ui_save(self,ar,**kw):
         if not self.state:
             self.state = TaskState.todo
+        return super(Task,self).before_ui_save(ar,**kw)
+        
+    #~ def on_user_change(self,request):
+        #~ if not self.state:
+            #~ self.state = TaskState.todo
         #~ self.user_modified = True
         
     @classmethod
@@ -1558,10 +1588,17 @@ class Guest(mixins.TypedPrintable,outbox.Mailable):
         
     @dd.displayfield(_("Event"))
     def event_summary(self,ar):
-        return ar.renderer.href_to(
-            self.event,event_summary(self.event,ar.get_user()))
+        return ar.href_to(self.event,event_summary(self.event,ar.get_user()))
         #~ return event_summary(self.event,ar.get_user())
         
+    def before_ui_save(self,ar,**kw):
+        if not self.state:
+            self.state = GuestState.invited
+        return super(Guest,self).before_ui_save(ar,**kw)
+        
+    #~ def on_user_change(self,request):
+        #~ if not self.state:
+            #~ self.state = GuestState.invited
         
 
     #~ def get_recipient(self):
@@ -1893,6 +1930,7 @@ def user_calendars(qs,user):
     #~ print 20120710, subs
     return qs.filter(id__in=subs)
 
+
 if settings.LINO.use_extensible:
   
     def parsedate(s):
@@ -1900,19 +1938,24 @@ if settings.LINO.use_extensible:
   
     class Panel(dd.Frame):
         required = dict(user_groups='office')
-        default_action = dd.Calendar()
+        default_action = CalendarAction()
         #~ default_action_class = dd.Calendar
 
     class PanelCalendars(Calendars):
         use_as_default_table = False
         required = dict(user_groups='office')
-        column_names = 'id name description color is_hidden'
+        #~ column_names = 'id name description color is_hidden'
+        column_names = 'id babel_name description color is_hidden'
         
         @classmethod
         def get_request_queryset(self,ar):
             qs = super(PanelCalendars,self).get_request_queryset(ar)
             return user_calendars(qs,ar.get_user())
             
+        @dd.displayfield()
+        def babel_name(cls,self,ar):
+            return babel.babelattr(self,'name')
+            
         @dd.virtualfield(models.BooleanField(_('Hidden')))
         def is_hidden(cls,self,ar):
             return False
@@ -2006,8 +2049,8 @@ def reminders(ar,days_back=None,days_forward=None,**kw):
     
     """
     user = ar.get_user()
-    Task = resolve_model('cal.Task')
-    Event = resolve_model('cal.Event')
+    #~ Task = resolve_model('cal.Task')
+    #~ Event = resolve_model('cal.Event')
     today = datetime.date.today()
     
     past = {}
@@ -2025,32 +2068,45 @@ def reminders(ar,days_back=None,days_forward=None,**kw):
         else:
             day.append(cmp)
             
-    #~ filterkw = { 'due_date__lte' : today }
-    filterkw = {}
+    #~ filterkw = {}
+    flt = models.Q()
     if days_back is not None:
-        filterkw.update({ 
-            'start_date__gte' : today - datetime.timedelta(days=days_back)
-            #~ 'dt_alarm__gte' : today - datetime.timedelta(days=days_back)
-        })
+        flt = flt & models.Q(start_date__gte = today - datetime.timedelta(days=days_back))
+        #~ filterkw.update({ 
+            #~ 'start_date__gte' : today - datetime.timedelta(days=days_back)
+        #~ })
     if days_forward is not None:
-        filterkw.update({ 
-            'start_date__lte' : today + datetime.timedelta(days=days_forward)
-            #~ 'dt_alarm__lte' : today + datetime.timedelta(days=days_forward)
-        })
+        flt = flt & models.Q(start_date__lte=today + datetime.timedelta(days=days_forward))
+        #~ filterkw.update({ 
+            #~ 'start_date__lte' : today + datetime.timedelta(days=days_forward)
+        #~ })
     #~ filterkw.update(dt_alarm__isnull=False)
-    filterkw.update(user=user)
+    #~ filterkw.update(user=user)
     
-    for o in Event.objects.filter(
-        #~ models.Q(status=None) | models.Q(status__reminder=True),
-        models.Q(state=None) | models.Q(state__lte=EventState.scheduled),
-        **filterkw).order_by('start_date'):
+    events = ar.spawn(MyEvents,
+        master_instance=user,
+        filter=flt & (models.Q(state=None) | models.Q(state__lte=EventState.scheduled)))
+    tasks = ar.spawn(MyTasks,master_instance=user,
+        filter=flt & models.Q(state__in=[TaskState.blank_item,TaskState.todo]))
+    
+    for o in events:
+        o._detail_action = MyEvents.detail_action
         add(o)
         
+    for o in tasks:
+        o._detail_action = MyTasks.detail_action
+        add(o)
+        
+    #~ for o in Event.objects.filter(
+        #~ models.Q(state=None) | models.Q(state__lte=EventState.scheduled),
+        #~ **filterkw).order_by('start_date'):
+        #~ add(o)
+        
     #~ filterkw.update(done=False)
-    filterkw.update(state__in=[TaskState.blank_item,TaskState.todo])
+    #~ filterkw.update(state__in=[TaskState.blank_item,TaskState.todo])
             
-    for task in Task.objects.filter(**filterkw).order_by('start_date'):
-        add(task)
+    #~ for task in Task.objects.filter(**filterkw).order_by('start_date'):
+        #~ add(task)
         
     def loop(lookup,reverse):
         sorted_days = lookup.keys()
@@ -2059,6 +2115,7 @@ def reminders(ar,days_back=None,days_forward=None,**kw):
             sorted_days.reverse()
         for day in sorted_days:
             yield '<h3>'+dtosl(day) + '</h3>'
+            #~ yield dd.summary(ar,lookup[day],**kw)
             yield dd.summary(ar,lookup[day],**kw)
             
     if days_back is not None:
@@ -2090,7 +2147,7 @@ class UpdateReminders(actions.RowAction):
     callable_from = (actions.GridEdit, actions.ShowDetailAction)
         
     def run(self,user,ar,**kw):
-        logger.info("Updating reminders for %s",user)
+        logger.info("Updating reminders for %s",unicode(user))
         n = update_reminders(user)
         kw.update(success=True)
         msg = _("%(num)d reminders for %(user)s have been updated."
@@ -2207,6 +2264,7 @@ def setup_main_menu(site,ui,user,m):
     #~ m  = m.add_menu("events",_("Events"))
     m.add_action(MyEvents)
     #~ m.add_action(MyEventsToday)
+    #~ m.add_action(MyEventsReserved)
     m.add_action(MyEventsSuggested)
     m.add_action(MyEventsNotified)
     #~ m.add_action(MyEventsToSchedule)
@@ -2215,6 +2273,7 @@ def setup_main_menu(site,ui,user,m):
     
     m.add_separator('-')
     m.add_action(Events)
+    #~ m.add_action(EventsReserved)
     m.add_action(EventsSuggested)
     m.add_action(EventsNotified)
     #~ m.add_action(EventsToSchedule)
@@ -2271,6 +2330,12 @@ def setup_quicklinks(site,ui,user,m):
         m.add_action(MyEventsNotified)
         m.add_action(MyTasksToDo)
         
+def whats_up(site,ui,user):
+    #~ MyEventsReserved
+    MyEventsSuggested
+    MyEventsNotified
+    
+        
 dd.add_user_group('office',MODULE_LABEL)
 
 customize_users()
\ No newline at end of file
