@@ -19,6 +19,10 @@ It defines tables like `Person` and `Company`
 
 """
 
+import logging
+logger = logging.getLogger(__name__)
+
+
 import datetime
 from dateutil.relativedelta import relativedelta
 
@@ -35,7 +39,6 @@ from django.utils import translation
 
 import lino
 #~ from lino import layouts
-from lino.utils import perms
 
 from lino import dd
 #~ from lino import fields
@@ -45,7 +48,7 @@ from lino.utils import join_words
 from lino.utils.choosers import chooser
 from lino.utils.choicelists import Gender
 from lino.utils import babel 
-from lino.models import get_site_config
+#~ from lino.models import get_site_config
 
 from lino.modlib.countries.models import CountryCity
 
@@ -98,6 +101,7 @@ class CompanyType(babel.BabelNamed):
     
         
 class CompanyTypes(dd.Table):
+    required = dict(user_level='manager')
     model = 'contacts.CompanyType'
     column_names = 'name *'
     #~ label = _("Company types")
@@ -105,7 +109,8 @@ class CompanyTypes(dd.Table):
 
 
 
-class Contact(mti.MultiTableBase,CountryCity):
+#~ class Contact(mti.MultiTableBase,CountryCity):
+class Partner(mti.MultiTableBase,CountryCity):
     """
     Base class for anything that has contact information 
     (postal address, email, phone,...).
@@ -113,15 +118,15 @@ class Contact(mti.MultiTableBase,CountryCity):
     """
     
     """
-    preferred width for ForeignKey fields to a Contact
+    preferred width for ForeignKey fields to a Partner
     """
     _lino_preferred_width = 20 
     
   
     class Meta:
         #~ abstract = True
-        verbose_name = _("Contact")
-        verbose_name_plural = _("Contacts")
+        verbose_name = _("Partner")
+        verbose_name_plural = _("Partners")
   
     name = models.CharField(max_length=200,verbose_name=_('Name'))
     addr1 = models.CharField(_("Address line before street"),
@@ -160,23 +165,24 @@ class Contact(mti.MultiTableBase,CountryCity):
     
     def save(self,*args,**kw):
         if self.id is None:
-            sc = get_site_config()
+            sc = settings.LINO.site_config # get_site_config()
             if sc.next_partner_id is not None:
                 self.id = sc.next_partner_id
                 sc.next_partner_id += 1
                 sc.save()
-        super(Contact,self).save(*args,**kw)
+        #~ logger.info("20120327 Partner.save(%s,%s)",args,kw)
+        super(Partner,self).save(*args,**kw)
         
     def __unicode__(self):
         return self.name
-
+        
     def address_person_lines(self):
         #~ yield self.name
         yield self.get_full_name()
         
     def get_full_name(self,*args,**kw):
         """\
-Returns a one-line string representing this Contact.
+Returns a one-line string representing this Partner.
 The default returns simply the `name` field, ignoring any parameters, 
 but e.g. :class:`PersonMixin` overrides this.
         """
@@ -216,7 +222,7 @@ but e.g. :class:`PersonMixin` overrides this.
         #~ else:
             #~ foreigner = (self.country != self.objects.get(pk=1).country)
         if self.country is not None:
-            sc = get_site_config()
+            sc = settings.LINO.site_config # get_site_config()
             if not sc.site_company or self.country != sc.site_company.country: 
                 # (if self.country != sender's country)
                 yield unicode(self.country)
@@ -253,27 +259,25 @@ but e.g. :class:`PersonMixin` overrides this.
     def name_column(self,request):
         #~ return join_words(self.last_name.upper(),self.first_name)
         return unicode(self)
-        #~ return self.get_full_name(nominative=True)
-    #~ name_column.return_type = dd.DisplayField(_("Name"))
     
 
-class ContactDetail(dd.DetailLayout):
+class PartnerDetail(dd.FormLayout):
   
     main = """
-    address_box contact_box
+    address_box:60 contact_box:30
     bottom_box
     """
     
     address_box = """
-    name 
-    country region
-    city zip_code:10
+    name_box
+    country region city zip_code:10
+    addr1
     street_prefix street:25 street_no street_box
-    addr2:40
+    addr2
     """
-
+    
     contact_box = """
-    id language
+    info_box
     email:40 
     url
     phone
@@ -282,9 +286,12 @@ class ContactDetail(dd.DetailLayout):
 
     bottom_box = """
     remarks 
-    is_person is_company is_user
+    is_person is_company #is_user
     """
         
+    name_box = "name"
+    info_box = "id language"
+    
     
     def setup_handle(self,dh):
         dh.address_box.label = _("Address")
@@ -292,37 +299,41 @@ class ContactDetail(dd.DetailLayout):
   
     
     
-class Contacts(dd.Table):
-    model = 'contacts.Contact'
+class Partners(dd.Table):
+    model = 'contacts.Partner'
     column_names = "name email * id" 
     order_by = ['name','id']
     #~ column_names = "name * id" 
-    detail_layout = ContactDetail()
+    detail_layout = PartnerDetail()
+    insert_layout = dd.FormLayout("""
+    name
+    language email
+    """,window_size=(40,'auto'))
     
     @classmethod
     def get_queryset(self):
         return self.model.objects.select_related('country','city')
 
 
-class AllContacts(Contacts):
+class AllPartners(Partners):
   
     @classmethod
-    def init_label(self):
+    def get_actor_label(self):
         return _("All %s") % self.model._meta.verbose_name_plural
         
-class ContactsByCity(Contacts):
+class PartnersByCity(Partners):
     master_key = 'city'
     order_by = 'street street_no street_box addr2'.split()
     column_names = "street street_no street_box addr2 name language *"
     
-class ContactsByCountry(Contacts):
+class PartnersByCountry(Partners):
     master_key = 'country'
     column_names = "city street street_no name language *"
     order_by = "city street street_no".split()
 
 
 
-class Born(models.Model):
+class Born(dd.Model):
     """
     Abstract base class that adds a `birth_date` 
     field and a virtual field "Age".
@@ -340,37 +351,54 @@ class Born(models.Model):
     #~ birth_date_circa = models.BooleanField(
         #~ default=False,
         #~ verbose_name=_("not exact"))
-        
-    def age(self,request,today=None):
+    
+    def get_age_years(self,today=None):
         if self.birth_date and self.birth_date.year:
             if today is None:
                 today = datetime.date.today()
             try:
-                dd = today - self.birth_date.as_date()
+                return today - self.birth_date.as_date()
             except ValueError:
                 pass
-            else:
-                s = _("%d years") % (dd.days / 365)
-                if self.birth_date.is_complete():
-                    return s
-                return u"±" + s
-        return _('unknown')
-    age.return_type = dd.DisplayField(_("Age"))
+      
+    @dd.displayfield(_("Age"))
+    def age(self,request,today=None):
+        a = self.get_age_years(today)
+        if a is None:
+            return unicode(_('unknown'))
+        s = _("%d years") % (a.days / 365)
+        if self.birth_date and self.birth_date.is_complete():
+            return s
+        return u"±" + s
+    
+    #~ @dd.displayfield(_("Age"))
+    #~ def age(self,request,today=None):
+        #~ if self.birth_date and self.birth_date.year:
+            #~ if today is None:
+                #~ today = datetime.date.today()
+            #~ try:
+                #~ dd = today - self.birth_date.as_date()
+            #~ except ValueError:
+                #~ pass
+            #~ else:
+                #~ s = _("%d years") % (dd.days / 365)
+                #~ if self.birth_date.is_complete():
+                    #~ return s
+                #~ return u"±" + s
+        #~ return _('unknown')
     
 
 
 #~ Note `PersonMixin` must not be named `Person` because users.User also inherits 
 #~ from it and would then also find all contacs/Person/*.dtl !
 
-class PersonMixin(models.Model):
+class PersonMixin(dd.Model):
     """
-    Mixin for models that represent a physical person. 
+    Can be used also for Persons that are no Partners
     """
     class Meta:
         abstract = True
-        verbose_name = _("Person")
-        verbose_name_plural = _("Persons")
-
+        
     first_name = models.CharField(max_length=200,
       #~ blank=True,
       verbose_name=_('First name'))
@@ -385,7 +413,6 @@ class PersonMixin(models.Model):
       verbose_name=_('Title'))
     """Text to print as part of the first address line in front of first_name."""
         
-    #~ gender = GenderField()
     gender = Gender.field()
         
     def get_salutation(self,**salutation_options):
@@ -399,7 +426,7 @@ class PersonMixin(models.Model):
         
 The optional keyword argument `salutation` can be set to `False` 
 to suppress salutations. 
-See :func:`lino.apps.dsbe.tests.dsbe_tests.test04` 
+See :func:`lino.apps.pcsw.tests.pcsw_tests.test04` 
 and
 :func:`lino.modlib.contacts.tests.test01` 
 for some examples.
@@ -415,6 +442,18 @@ Optional `salutation_options` see :func:`get_salutation`.
     full_name = property(get_full_name)
     #~ full_name.return_type = models.CharField(max_length=200,verbose_name=_('Full name'))
     
+  
+#~ class Person(dd.Model):
+class Person(Partner,PersonMixin):
+    """
+    Mixin for models that represent a physical person. 
+    """
+    class Meta:
+        #~ abstract = True
+        abstract = settings.LINO.is_abstract_model('contacts.Person')
+        verbose_name = _("Person")
+        verbose_name_plural = _("Persons")
+
     def address_person_lines(self,*args,**kw):
         "Deserves more documentation."
         if self.title:
@@ -424,46 +463,39 @@ Optional `salutation_options` see :func:`get_salutation`.
         #~ yield  " ".join(l)
         
     def full_clean(self,*args,**kw):
-        l = filter(lambda x:x,[self.last_name,self.first_name])
-        self.name = " ".join(l)
-        super(PersonMixin,self).full_clean(*args,**kw)
+        """
+        Set the :attr:`Partner.name` field of this person. 
+        This field is visible in the Partner's detail but not 
+        in the Person's detail and serves for sorting 
+        when selecting a Partner. 
+        It also serves for quick search on Persons.
+        """
+        #~ l = filter(lambda x:x,[self.last_name,self.first_name])
+        #~ self.name = " ".join(l)
+        self.name = join_words(self.last_name,self.first_name)
+        super(Person,self).full_clean(*args,**kw)
 
 
 
 
 
-class PersonDetail(dd.DetailLayout):
+class PersonDetail(PartnerDetail):
   
-    main = """
-    address_box contact_box
-    bottom_box RolesByPerson
-    """
+    #~ main = """
+    #~ address_box contact_box
+    #~ bottom_box contacts.RolesByPerson
+    #~ """
     
-    address_box = """
-    last_name first_name:15 title:10
-    country region
-    city zip_code:10
-    street_prefix street:25 street_no street_box
-    addr2:40
-    """
+    name_box = "last_name first_name:15 gender title:10"
+    info_box = "id:5 language:10 birth_date:10"
 
-    contact_box = """
-    id language
-    email:40 
-    url
-    phone
-    gsm fax
-    """
-
-    bottom_box = """
-    gender birth_date age
-    remarks
-    """
+    bottom_box = "remarks contacts.RolesByPerson"
         
     
-    def setup_handle(self,dh):
-        dh.address_box.label = _("Address")
-        dh.contact_box.label = _("Contact")
+    #~ def setup_handle(self,dh):
+        #~ PartnerDetail.setup_handle(self,dh)
+        #~ dh.address_box.label = _("Address")
+        #~ dh.contact_box.label = _("Contact")
   
 
 
@@ -476,16 +508,22 @@ class Persons(dd.Table):
     column_names = "name_column:20 address_column email phone:10 gsm:10 id language:10 *"
     detail_layout = PersonDetail()
     
+    insert_layout = dd.FormLayout("""
+    title first_name last_name
+    gender language
+    """,window_size=(60,'auto'))
+    
 
 
-class CompanyMixin(models.Model):
-#~ class Company(Contact):
+#~ class CompanyMixin(dd.Model):
+class Company(Partner):
     """
     Abstract base class for a company.
     See also :doc:`/tickets/14`.
     """
     class Meta:
-        abstract = True
+        abstract = settings.LINO.is_abstract_model('contacts.Company')
+        #~ abstract = True
         app_label = 'contacts'
         verbose_name = _("Company")
         verbose_name_plural = _("Companies")
@@ -513,45 +551,24 @@ class CompanyMixin(models.Model):
         #~ raise Exception('20110810')
 
 
-class CompanyDetail(dd.DetailLayout):
+class CompanyDetail(PartnerDetail):
   
-    box3 = """
-    country region
-    city zip_code:10
-    street_prefix street:25 street_no street_box
-    addr2:40
-    """
-
-    box4 = """
-    email:40 
-    url
-    phone
-    gsm
-    """
-
-    address_box = "box3 box4"
-
     bottom_box = "remarks contacts.RolesByCompany"
 
-    intro_box = """
-    prefix name id language 
-    vat_id:12 type:20
-    """
-
-    main = """
-    intro_box
-    address_box
-    bottom_box
-    """
-    
+    name_box = """prefix name type:20"""
+    info_box = "id:5 language:10 vat_id:12"
 
 
 
               
-class Companies(Contacts):
+class Companies(Partners):
     model = settings.LINO.company_model
     order_by = ["name"]
     detail_layout = CompanyDetail()
+    insert_layout = dd.FormLayout("""
+    name type
+    language email
+    """,window_size=(40,'auto'))
     
     
     
@@ -569,12 +586,13 @@ class RoleType(babel.BabelNamed):
 
 
 class RoleTypes(dd.Table):
+    required = dict(user_level='manager')
     model = 'contacts.RoleType'
 
 
-#~ class Contact(models.Model):
-#~ class RoleOccurence(models.Model):
-class Role(models.Model):
+#~ class Contact(dd.Model):
+#~ class RoleOccurence(dd.Model):
+class Role(dd.Model):
 #~ class unused_Role(object):
     """
     The role of a given :class:`Person` in a given :class:`Company`.
@@ -646,56 +664,65 @@ class Role(models.Model):
     #~ column_names = 'company type *'
     
 class Roles(dd.Table):
+    required = dict(user_level='manager')
+    #~ required_user_level = UserLevels.manager
     model = 'contacts.Role'   
     
 class RolesByCompany(Roles):
+    required = dict()
+    #~ required_user_level = None
     label = _("Contact persons")
     master_key = 'company'
     column_names = 'person type *'
 
 class RolesByPerson(Roles):
+    required = dict()
+    #~ required_user_level = None
     label = _("Contact for")
     master_key = 'person'
     column_names = 'company type *'
     
     
     
-class PartnerDocument(models.Model):
+class PartnerDocument(dd.Model):
     """
-    Maybe deprecated. 
-    This adds two fields 'person' and 'company' to this model, 
+    Adds two fields 'partner' and 'person' to this model, 
     making it something that refers to a "partner". 
-    If `company` is empty, the "partner" is a private person.
-    If `company` is filled, then `person` means a "contact person" 
-    for this company.
+    `person` means a "contact person" for the partner.
     
     """
     
     class Meta:
         abstract = True
         
+    company = models.ForeignKey(settings.LINO.company_model,
+        blank=True,null=True,
+        #~ verbose_name=_("Company")
+        )
+        
     #~ person = models.ForeignKey("contacts.Person",
     person = models.ForeignKey(settings.LINO.person_model,
         blank=True,null=True,
         #~ verbose_name=_("Person")
         )
-    company = models.ForeignKey(settings.LINO.company_model,
-        blank=True,null=True,
-        #~ verbose_name=_("Company")
-        )
         
     def get_partner(self):
         if self.company is not None:
             return self.company
         return self.person
         
-    def get_mailable_contacts(self):
+    def get_mailable_recipients(self):
         for p in self.company, self.person:
             if p is not None and p.email:
                 #~ yield "%s <%s>" % (p, p.email)
                 yield ('to', p)
                 #~ yield ('to', unicode(p), p.email)
         
+    def get_postable_recipients(self):
+        for p in self.company, self.person:
+            if p is not None:
+                yield p
+        
         
     #~ def summary_row(self,ui,rr,**kw):
         #~ if self.person:
@@ -708,88 +735,98 @@ class PartnerDocument(models.Model):
             #~ return ui.href_to(self.company)
             
     #~ def summary_row(self,ui,rr,**kw):
-    def summary_row(self,ui,**kw):
-        s = ui.href_to(self)
+    def summary_row(self,ar,**kw):
+        """
+        A :modmeth:`summary_row` method for partner documents.
+        """
+        href_to = ar.href_to
+        #~ href_to = ui.ext_renderer.href_to
+        s = href_to(self)
         #~ if self.person and not dd.has_fk(rr,'person'):
         if self.person:
             if self.company:
-                s += " (" + ui.href_to(self.person) + "/" + ui.href_to(self.company) + ")"
+                s += " (" + href_to(self.person) \
+                    + "/" + href_to(self.company) + ")"
             else:
-                s += " (" + ui.href_to(self.person) + ")"
+                s += " (" + href_to(self.person) + ")"
         elif self.company:
-            s += " (" + ui.href_to(self.company) + ")"
+            s += " (" + href_to(self.company) + ")"
         return s
             
-    def update_owned_instance(self,task):
-        task.person = self.person
-        task.company = self.company
+    def update_owned_instance(self,other):
+        #~ print '20120627 PartnerDocument.update_owned_instance'
+        if isinstance(other,mixins.ProjectRelated):
+            # the following hack doesn't work when loading data by dumpy
+            # because LINO.person_model are still strings at that moment
+            if isinstance(self.person,settings.LINO.person_model):
+                other.project = self.person
+            elif isinstance(self.company,settings.LINO.person_model):
+                other.project = self.company
+        other.person = self.person
+        other.company = self.company
+        super(PartnerDocument,self).update_owned_instance(other)
         
-class ContactDocument(models.Model):
-    """
-    A document whose recipient is a :class:`Contact`.
-    """
-  
-    class Meta:
-        abstract = True
-        
-    contact = models.ForeignKey("contacts.Contact",
-        #~ blank=True,null=True,
-        related_name="%(app_label)s_%(class)s_by_contact",
-        #~ related_name="%(app_label)s_%(class)s_related",
-        verbose_name=_("Contact"))
-    language = babel.LanguageField(default=babel.DEFAULT_LANGUAGE)
-
-    def get_mailable_contacts(self):
-        yield ('to',self.contact)
-
-    def get_recipient(self):
-        return self.contact
-    recipient = property(get_recipient)
-
 
 
     
 
 
-if settings.LINO.is_installed('contacts'):
+#~ if settings.LINO.is_installed('contacts'):
   
-    """
-    Don't inject fields if contacts is just being imported from some other module.
-    """
-  
-    from lino.models import SiteConfig
+    #~ """
+    #~ Don't inject fields if contacts is just being imported from some other module.
+    #~ """
+    
+#~ dd.inject_field(settings.LINO.user_model,
+    #~ 'partner',
+    #~ models.ForeignKey(Partner,
+        #~ blank=True,null=True,
+        #~ verbose_name=_("Partner")))
 
-    dd.inject_field(SiteConfig,
-        'next_partner_id',
-        models.IntegerField(default=100, # first 100 for users from demo fixtures.
-            verbose_name=_("The next automatic id for Person or Company")
-        ),"""The next automatic id for Person or Company. 
-        Deserves more documentation.
-        """)
-        
-    dd.inject_field(SiteConfig,
-        'site_company',
-        models.ForeignKey(settings.LINO.company_model,
-            blank=True,null=True,
-            verbose_name=_("The company that runs this site"),
-            related_name='site_company_sites',
-            ),
-        """The Company to be used as sender in documents.""")
-        
 
-    dd.inject_field(Contact,
-        'is_person',
-        #~ mti.EnableChild('contacts.Person',verbose_name=_("is Person")),
-        mti.EnableChild(settings.LINO.person_model,verbose_name=_("is Person")),
-        """Whether this Contact is also a Person."""
-        )
-    dd.inject_field(Contact,
-        'is_company',
-        mti.EnableChild(settings.LINO.company_model,verbose_name=_("is Company")),
-        """Whether this Contact is also a Company."""
+
+from lino.models import SiteConfig
+
+dd.inject_field(SiteConfig,
+    'next_partner_id',
+    models.IntegerField(default=100, # first 100 for users from demo fixtures.
+        verbose_name=_("The next automatic id for Person or Company")
+    ),"""The next automatic id for Person or Company. 
+    Deserves more documentation.
+    """)
+    
+dd.inject_field(SiteConfig,
+    'site_company',
+    models.ForeignKey(settings.LINO.company_model,
+        blank=True,null=True,
+        verbose_name=_("The company that runs this site"),
+        related_name='site_company_sites',
+        help_text="""\
+The Company to be used as sender in documents."""
         )
+    )
+    
+
+dd.inject_field(Partner,
+    'is_person',
+    #~ mti.EnableChild('contacts.Person',verbose_name=_("is Person")),
+    mti.EnableChild(settings.LINO.person_model,verbose_name=_("is Person")),
+    """Whether this Partner is a Person."""
+    )
+dd.inject_field(Partner,
+    'is_company',
+    mti.EnableChild(settings.LINO.company_model,verbose_name=_("is Company")),
+    """Whether this Partner is a Company."""
+    )
 
 
+MODULE_NAME = _("Contacts")
+
+def site_setup(site):
+    site.modules.countries.Cities.set_detail_layout("""
+    name country 
+    contacts.PartnersByCity
+    """)
 
 
 def setup_main_menu(site,ui,user,m): pass
@@ -797,8 +834,14 @@ def setup_main_menu(site,ui,user,m): pass
 def setup_my_menu(site,ui,user,m): 
     pass
   
+def setup_master_menu(site,ui,user,m): 
+    m = m.add_menu("contacts",MODULE_NAME)
+    m.add_action(Persons)
+    m.add_action(Companies)
+    m.add_action(Partners)
+    
 def setup_config_menu(site,ui,user,m): 
-    config_contacts = m.add_menu("contacts",_("Contacts"))
+    config_contacts = m.add_menu("contacts",MODULE_NAME)
     config_contacts.add_action(CompanyTypes)
     config_contacts.add_action(RoleTypes)
     config_contacts.add_action(site.modules.countries.Countries)
@@ -809,6 +852,11 @@ def setup_config_menu(site,ui,user,m):
     #~ m.add_action('contacts.RoleTypes')
   
 def setup_explorer_menu(site,ui,user,m):
-    config_contacts = m.add_menu("contacts",_("Contacts"))
+    m = m.add_menu("contacts",MODULE_NAME)
     m.add_action(site.modules.contacts.Roles)
+    m.add_action(site.modules.countries.Cities)
+  
+#~ def setup_quicklinks(site,ui,user,m):
+    #~ m.add_action(Person.detail_action)
+        
   
\ No newline at end of file
