@@ -13,7 +13,7 @@
 ## along with Lino; if not, see <http://www.gnu.org/licenses/>.
 
 """
-See also :doc:`/dsbe/models`.
+See also :doc:`/pcsw/models`.
 
 """
 import logging
@@ -37,12 +37,12 @@ from django.utils.encoding import force_unicode
 #~ from django.utils import translation
 
 
-
+ONE_DAY = datetime.timedelta(days=1)
 
 
 from lino import dd
 #~ from lino import layouts
-from lino.utils import perms
+from lino.core import perms
 from lino.utils import dblogger
 #~ from lino.utils import printable
 from lino import mixins
@@ -50,36 +50,36 @@ from lino.modlib.contacts import models as contacts
 from lino.modlib.notes import models as notes
 #~ from lino.modlib.links import models as links
 from lino.modlib.uploads import models as uploads
-from lino.utils.choicelists import HowWell
 #~ from lino.modlib.properties.utils import KnowledgeField #, StrengthField
 #~ from lino.modlib.uploads.models import UploadsByPerson
-from lino.models import get_site_config
-from lino.tools import get_field
-from lino.tools import resolve_field
-from lino.tools import range_filter
+from lino.core.modeltools import get_field
+from lino.core.modeltools import resolve_field
+from lino.core.modeltools import range_filter
 from lino.utils.babel import DEFAULT_LANGUAGE, babelattr, babeldict_getitem, language_choices
 from lino.utils.htmlgen import UL
 #~ from lino.utils.babel import add_babel_field, DEFAULT_LANGUAGE, babelattr, babeldict_getitem
 from lino.utils import babel 
 from lino.utils.choosers import chooser
 from lino.utils.choicelists import ChoiceList
+#~ from lino.utils.choicelists import UserLevel
 from lino.utils import mti
 from lino.mixins.printable import DirectPrintAction
 #~ from lino.mixins.reminder import ReminderEntry
-from lino.tools import obj2str
+from lino.core.modeltools import obj2str
 
 from lino.modlib.countries.models import CountryCity
-from lino.modlib.cal.models import DurationUnit
+from lino.modlib.cal.utils import DurationUnits
 
 # not used here, but these modules are required in INSTALLED_APPS, 
 # and other code may import them using 
-# ``from lino.apps.dsbe.models import Property``
+# ``from lino.apps.pcsw.models import Property``
 
 from lino.modlib.properties.models import Property
 from lino.modlib.notes.models import NoteType
 from lino.modlib.countries.models import Country, City
 from lino.modlib.isip.models import ContractBase
-from lino.apps.dsbe.models import Company, Companies
+#~ from lino.apps.pcsw.models import Company, Companies, CompanyDetail
+from lino.apps.pcsw import models as pcsw
 
 
 #~ SCHEDULE_CHOICES = {
@@ -127,8 +127,13 @@ class Schedule(babel.BabelNamed):
         verbose_name_plural = _('Work Schedules')
         
 class Schedules(dd.Table):
+    required = dict(user_groups='integ',user_level='manager')
     model = Schedule
     order_by = ['name']
+    detail_template = """
+    id name
+    ContractsBySchedule
+    """
 
 class Regime(babel.BabelNamed):
     """List of choices for `jobs.Contract.regime` field."""
@@ -137,13 +142,20 @@ class Regime(babel.BabelNamed):
         verbose_name_plural = _('Work Regimes')
         
 class Regimes(dd.Table):
+    required = dict(user_groups='integ',user_level='manager')
+    #~ required_user_groups = ['integ']
+    #~ required_user_level = UserLevels.manager
     model = Regime
     order_by = ['name']
+    detail_template = """
+    id name
+    ContractsByRegime
+    """
 
 
 
 
-class JobProvider(Company):
+class JobProvider(pcsw.Company):
     """Stellenanbieter (BISA, BW, ...) 
     """
     class Meta:
@@ -151,14 +163,42 @@ class JobProvider(Company):
         verbose_name = _("Job Provider")
         verbose_name_plural = _('Job Providers')
     
+    def disable_delete(self,ar):
+        pass
 
-class JobProviders(Companies):
+class JobProviderDetail(pcsw.CompanyDetail):
+    """
+    This is the same as CompanyDetail, except that we
+    
+    - remove MTI fields from `remark` panel
+    - add a new tab `jobs`
+    
+    """
+    box5 = "remarks" 
+    jobs = """
+    JobsByProvider
+    ContractsByProvider
+    """
+    main = "general notes jobs"
+    
+    def setup_handle(self,lh):
+        pcsw.CompanyDetail.setup_handle(self,lh)
+        lh.jobs.label = _("Jobs")
+
+  
+
+
+class JobProviders(pcsw.Companies,dd.Table):
     """
     List of Companies that have `Company.is_jobprovider` activated.
     """
-    use_as_default_report = False
+    required = dict(user_groups='integ')
+    #~ required_user_groups = ['integ']
+    #~ required_user_level = UserLevels.manager
+    #~ use_as_default_table = False
     model = JobProvider
     app_label = 'jobs'
+    detail_layout = JobProviderDetail()
   
 
 
@@ -171,7 +211,7 @@ class ContractType(mixins.PrintableType,babel.BabelNamed):
     :class:`lino.modlib.isip.models.ContractType` (see there 
     for general documentation).
     
-    They are separated tables because ISIP contracts are by nature 
+    They are separated tables because ISIP contracts are in practice
     very different from JOBS contracts, and also their types should 
     not be mixed.
     
@@ -192,8 +232,16 @@ class ContractType(mixins.PrintableType,babel.BabelNamed):
         
 
 class ContractTypes(dd.Table):
+    required = dict(user_groups='integ',user_level='manager')
+    #~ required_user_groups = ['integ']
+    #~ required_user_level = UserLevels.manager
     model = ContractType
     column_names = 'name ref build_method template *'
+    detail_template = """
+    id name 
+    ref build_method template
+    ContractsByType
+    """
 
 
 class Sector(babel.BabelNamed):
@@ -207,8 +255,16 @@ class Sector(babel.BabelNamed):
         verbose_name=_("Remark"))
         
 class Sectors(dd.Table):
+    required = dict(user_groups='integ',user_level='manager')
+    #~ required_user_groups = ['integ']
+    #~ required_user_level = UserLevels.manager
     model = Sector
     order_by = ['name']
+    detail_template = """
+    id name
+    remark FunctionsBySector
+    CandidaturesBySector
+  """
 
 class Function(babel.BabelNamed):
     """Each Job may have a Function."""
@@ -224,10 +280,20 @@ class Function(babel.BabelNamed):
         #~ related_name="%(app_label)s_%(class)s_set_by_provider",
         #~ verbose_name=_("Job Provider"),
         #~ blank=True,null=True)
+        
 class Functions(dd.Table):
+    required = dict(user_groups='integ',user_level='manager')
+    #~ required_user_groups = ['integ']
+    #~ required_user_level = UserLevels.manager
     model = Function
     column_names = 'name sector *'
     order_by = ['name']
+    detail_template = """
+    id name sector
+    remark
+    CandidaturesByFunction
+    ExperiencesByFunction
+    """
     
 class FunctionsBySector(Functions):
     master_key = 'sector'
@@ -239,7 +305,7 @@ class Contract(ContractBase):
     """
     A Contract
     
-    [NOTE1] I applies_from and duration are set, then the default value 
+    [NOTE1] If applies_from and duration are set, then the default value 
     for applies_until is computed 26 workdays per month:
     
     - duration `312` -> 12 months 
@@ -255,13 +321,13 @@ class Contract(ContractBase):
         related_name="%(app_label)s_%(class)s_set_by_type",
         verbose_name=_("Contract Type"),blank=True)
     
-    provider = models.ForeignKey(JobProvider,
-        related_name="%(app_label)s_%(class)s_set_by_provider",
-        #~ verbose_name=_("Employer"),
-        blank=True,null=True)
-    job = models.ForeignKey("jobs.Job",
-        verbose_name=_("Job"),
-        blank=True,null=True)
+    #~ provider = models.ForeignKey(JobProvider,
+        #~ related_name="%(app_label)s_%(class)s_set_by_provider",
+        #~ blank=True,null=True)
+    job = models.ForeignKey("jobs.Job")
+    #~ job = models.ForeignKey("jobs.Job",
+        #~ verbose_name=_("Job"),
+        #~ blank=True,null=True)
         
     duration = models.IntegerField(_("duration (days)"),
         blank=True,null=True,default=None)
@@ -289,29 +355,30 @@ class Contract(ContractBase):
     #~ aid_nature = models.CharField(_("aid nature"),max_length=100,blank=True)
     #~ aid_rate = models.CharField(_("aid rate"),max_length=100,blank=True)
     
-    @chooser()
-    def contact_choices(cls,provider):
-        if provider is not None:
-            #~ return provider.rolesbyparent.all()
-            #~ return provider.rolesbyparent.filter(type__use_in_contracts=True)
-            #~ return links.Link.objects.filter(type__use_in_contracts=True,a=provider)
-            return contacts.Role.objects.filter(
-                type__use_in_contracts=True,company=provider)
-        return []
+    #~ @chooser()
+    #~ def contact_choices(cls,provider):
+        #~ if provider is not None:
+            #~ return contacts.Role.objects.filter(
+                #~ type__use_in_contracts=True,company=provider)
+        #~ return []
         
-    def get_company(self):
-        return self.provider
-    company = property(get_company)
-    """for backwards compatibility. Document templates use a field `company`.
-    """
+    @chooser()
+    def company_choices(cls):
+        return JobProvider.objects.all()
+        
+    #~ def get_company(self):
+        #~ return self.provider
+    #~ company = property(get_company)
+    #~ """for backwards compatibility. Document templates use a field `company`.
+    #~ """
 
-    def get_recipient(self):
-        if self.contact:
-            return self.contact
-        if self.provider:
-            return self.provider
-        return self.person
-    recipient = property(get_recipient)
+    #~ def get_recipient(self):
+        #~ if self.contact:
+            #~ return self.contact
+        #~ if self.provider:
+            #~ return self.provider
+        #~ return self.person
+    #~ recipient = property(get_recipient)
     
     
     #~ def prepare_printable(self):
@@ -341,41 +408,17 @@ class Contract(ContractBase):
         ]
     
     
-    def disabled_fields(self,request):
+    def disabled_fields(self,ar):
         df = []
-        if self.job:
+        if self.job_id is not None:
             if self.job.provider:
-                df.append('provider')
+                df.append('company')
             if self.job.contract_type:
                 df.append('type')
         if not self.build_time:
             return df 
         return df + self.PRINTABLE_FIELDS
         
-    def unused_get_reminder_html(self,ui,user):
-        #~ url = ui.get_detail_url(self,fmt='detail')
-        #~ if self.type:
-            #~ s = unicode(self.type)
-        #~ else:
-            #~ s = self._meta.verbose_name
-        #~ s += ' #' + unicode(self.pk)
-        
-        #~ s = ui.href(url,cgi.escape(s))
-        s = ui.href_to(self)
-        
-        more = []
-        if self.person:
-            more.append(ui.href_to(self.person))
-        if self.provider:
-            more.append(ui.href_to(self.provider))
-        if self.user and self.user != user:
-            more.append(cgi.escape(unicode(self.user)))
-        if self.reminder_text:
-            more.append(cgi.escape(self.reminder_text))
-        else:
-            more.append(cgi.escape(_('Due date reached')))
-        return s + '&nbsp;: ' + (', '.join(more))
-        
     #~ def save(self,*args,**kw):
         #~ if self.job_id is not None:
             #~ qs = self.person.candidature_set.filter(contract=self)
@@ -427,23 +470,24 @@ class Contract(ContractBase):
                             self.applies_until = self.applies_from + datetime.timedelta(days=self.duration)
                             
                 if self.duration and not self.applies_until:
+                    # [NOTE1]
                     #~ self.applies_until = self.applies_from + datetime.timedelta(days=self.duration)
-                    self.applies_until = DurationUnit.months.add_duration(
-                      self.applies_from,self.duration/26)  # [NOTE1]
+                    self.applies_until = DurationUnits.months.add_duration(
+                      self.applies_from,self.duration/26) - ONE_DAY
               
         #~ if self.job_id is not None:
-        if self.job:
+        if self.job_id is not None:
             if self.job.provider is not None:
-                self.provider = self.job.provider
+                self.company = self.job.provider
             if self.job.contract_type is not None:
                 self.type = self.job.contract_type
             if self.hourly_rate is None:
                 self.hourly_rate = self.job.hourly_rate
             
-        if self.provider is not None:
-            if self.contact is not None:
-                if self.contact.company.pk != self.provider.company_ptr.pk:
-                    self.contact = None
+        #~ if self.company is not None:
+            #~ if self.contact is not None:
+                #~ if self.contact.company.pk != self.company.company_ptr.pk:
+                    #~ self.contact = None
                     
         super(Contract,self).full_clean(*args,**kw)
         
@@ -456,7 +500,7 @@ class Contract(ContractBase):
         #~ resolve_field('jobs.Contract.user').verbose_name=_("responsible (DSBE)")
         #~ lino.CONTRACT_PRINTABLE_FIELDS = dd.fields_list(cls,
         cls.PRINTABLE_FIELDS = dd.fields_list(cls,
-            'person job provider contact type '
+            'person job company contact type '
             'applies_from applies_until duration '
             'language schedule regime hourly_rate refund_rate '
             'reference_person responsibilities '
@@ -464,11 +508,48 @@ class Contract(ContractBase):
             'date_decided date_issued ')
         #~ super(Contract,cls).site_setup(lino)
 
+class ContractDetail(dd.FormLayout):
+    box1 = """
+    id:8 person:25 user:15 user_asd:15 language:8
+    job type company contact:20     
+    applies_from duration applies_until exam_policy
+    regime:20 schedule:30 hourly_rate:10 refund_rate:10
+    date_decided date_issued date_ended ending
+    reference_person build_time
+    responsibilities 
+    """
+    
+    right = """
+    cal.EventsByController 
+    cal.TasksByController 
+    """
+    
+    main = """
+    box1:70 right:30
+    """
+    
+  
 class Contracts(dd.Table):
+    required = dict(user_groups='integ')
+    #~ required_user_groups = ['integ']
+    #~ required_user_level = UserLevels.manager
     model = Contract
     column_names = 'id job applies_from applies_until user type *'
     order_by = ['id']
-    active_fields = 'job provider contact'.split()
+    active_fields = 'job company contact'.split()
+    
+    #~ detail_template = """
+    #~ id:8 person:25 user:15 user_asd:15 language:8
+    #~ job type company contact:20     
+    #~ applies_from duration applies_until 
+    #~ regime:20 schedule:30 hourly_rate:10 refund_rate:10
+    #~ date_decided date_issued date_ended ending
+    #~ reference_person build_time
+    #~ responsibilities cal.TasksByController cal.EventsByController 
+    #~ """
+    detail_layout = ContractDetail()
+    
+    
     
 class ContractsByPerson(Contracts):
     master_key = 'person'
@@ -476,7 +557,7 @@ class ContractsByPerson(Contracts):
 
         
 class ContractsByProvider(Contracts):
-    master_key = 'provider'
+    master_key = 'company'
     column_names = 'person job applies_from applies_until user type *'
 
 class ContractsByType(Contracts):
@@ -489,6 +570,10 @@ class ContractsByJob(Contracts):
     master_key = 'job'
 
 class ContractsByRegime(Contracts):
+    """
+    Shows Job Contracts for a given Regime.
+    Used as slave grid in Regimes detail.
+    """
     master_key = 'regime'
     column_names = 'job applies_from applies_until user type *'
 
@@ -496,9 +581,9 @@ class ContractsBySchedule(Contracts):
     master_key = 'schedule'
     column_names = 'job applies_from applies_until user type *'
 
-class MyContracts(mixins.ByUser,Contracts):
+class MyContracts(Contracts,mixins.ByUser):
     column_names = "applies_from person job *"
-    label = _("My contracts")
+    #~ label = _("My contracts")
     #~ order_by = "reminder_date"
     #~ column_names = "reminder_date person company *"
     order_by = ["applies_from"]
@@ -530,7 +615,7 @@ class JobType(mixins.Sequenced):
         return unicode(self.name)
         
   
-class SectorFunction(models.Model):
+class SectorFunction(dd.Model):
     """
     Abstract base for models that refer to a 
     :class:`Sector` and a :class:`Function`.
@@ -581,7 +666,16 @@ class Offer(SectorFunction):
         return u'%s @ %s' % (self.function,self.provider)
   
 class Offers(dd.Table):
+    required = dict(user_groups='integ')
+    #~ required_user_groups = ['integ']
+    #~ required_user_level = UserLevels.manager
     model = Offer
+    detail_template = """
+    name provider sector function
+    selection_from selection_until start_date
+    remark 
+    ExperiencesByOffer CandidaturesByOffer
+    """
     
     
     
@@ -661,13 +755,19 @@ class StudiesByCountry(Studies):
     master_key = 'country'
     
 class StudiesByCity(Studies):
+    """
+    Lists all Studies in a given City. 
+    Used as slave grid in Cities detail.
+    """
     master_key = 'city'
+    column_names = 'school type person content started stopped success language  remarks *'
+    
     
 class StudiesByPerson(HistoryByPerson):
-    "List of studies for a known person."
+    "List of studies for a given person."
     model = Study
     #~ label = _("Studies & experiences")
-    button_label = _("Studies")
+    #~ button_label = _("Studies")
     column_names = 'type content started stopped country city success language school remarks *'
     
 class Experience(SectorFunction):
@@ -747,7 +847,7 @@ class Job(SectorFunction):
             return u'%s bei %s' % (self.name,self.provider.name)
         return self.name
   
-    def disabled_fields(self,request):
+    def disabled_fields(self,ar):
         #~ if self.contract_set.count():
         #~ if self.contract_set.filter(must_build=False).count():
         if self.contract_set.filter(build_time__isnull=False).count():
@@ -854,7 +954,7 @@ class Candidature(SectorFunction):
     
     #~ date_submitted = models.DateField(_("date submitted"),auto_now_add=True)
     date_submitted = models.DateField(_("date submitted"))
-    u"Das Datum, an dem die Anfrage erstellt wurde."
+    u"Datum, an dem die Anfrage erstellt wurde."
     
     #~ contract = models.ForeignKey("jobs.Contract",blank=True,null=True,
         #~ verbose_name=_("Contract found"))
@@ -885,16 +985,25 @@ class Candidature(SectorFunction):
                   #~ "Cannot satisfy a Candidature with a Contract on another Person")
         #~ super(Candidature,self).clean(*args,**kw)
     
-    def on_create(self,req):
+    def on_create(self,ar):
         self.date_submitted = datetime.date.today()
+        super(Candidature,self).on_create(ar)
     
 
 class Candidatures(dd.Table):
+    """
+    List of :class:`Candidatures <Candidature>`.
+    """
+    required = dict(user_groups='integ',user_level='manager')
+    #~ required_user_groups = ['integ']
+    #~ required_user_level = UserLevels.manager
     model = Candidature
     order_by = ['date_submitted']
     column_names = 'date_submitted job:25 * id'
 
 class CandidaturesByPerson(Candidatures):
+    required = dict(user_groups='integ')
+    #~ required_user_level = None
     master_key = 'person'
 
 class CandidaturesBySector(Candidatures):
@@ -904,6 +1013,8 @@ class CandidaturesByFunction(Candidatures):
     master_key = 'function'
 
 class CandidaturesByJob(Candidatures):
+    required = dict(user_groups='integ')
+    #~ required_user_level = None
     master_key = 'job'
     column_names = 'date_submitted person:25 * id'
   
@@ -926,8 +1037,8 @@ class SectorFunctionByOffer(dd.Table):
     which is allowed only because it overrides
     :meth:`get_request_queryset`.
     """
-    can_add = perms.never
-    can_change = perms.never
+    #~ can_add = perms.never
+    #~ can_change = perms.never
     master = Offer
     
     @classmethod
@@ -976,13 +1087,32 @@ class ExperiencesByOffer(SectorFunctionByOffer):
 
 
 class Jobs(dd.Table):
+    required = dict(user_groups='integ')
+    #~ required_user_groups = ['integ']
+    #~ required_user_level = UserLevels.manager
     model = Job
     #~ order_by = ['start_date']
     column_names = 'name provider * id'
+    
+    detail_template = """
+    name provider contract_type type id 
+    sector function capacity hourly_rate 
+    remark CandidaturesByJob
+    ContractsByJob
+    """
+
+    
 
 class JobTypes(dd.Table):
+    required = dict(user_groups='integ',user_level='manager')
+    #~ required_user_groups = ['integ']
+    #~ required_user_level = UserLevels.manager
     model = JobType
     order_by = ['name']
+    detail_template = """
+    id name 
+    JobsByType
+    """
 
 class JobsByProvider(Jobs):
     master_key = 'provider'
@@ -1003,8 +1133,8 @@ class ContractsByType(Contracts):
   
 if True: # settings.LINO.user_model:
   
-    from lino.tools import resolve_model, UnresolvedModel
-    USER_MODEL = resolve_model(settings.LINO.user_model)
+    from lino.core.modeltools import resolve_model, UnresolvedModel
+    #~ USER_MODEL = resolve_model(settings.LINO.user_model)
     
     class ContractsSearch(Contracts):
         """
@@ -1012,10 +1142,10 @@ if True: # settings.LINO.user_model:
         """
         label = _("Job Contracts Search")
         
-        use_as_default_report = False
+        use_as_default_table = False
         
         parameters = dict(
-          user = models.ForeignKey(USER_MODEL,blank=True),
+          user = dd.ForeignKey(settings.LINO.user_model,blank=True),
           #~ user = models.ForeignKey(settings.LINO.user_model,blank=True),
           type = models.ForeignKey(ContractType,blank=True,verbose_name=_("Only contracts of type")),
           show_past = models.BooleanField(_("past contracts"),default=True),
@@ -1033,14 +1163,14 @@ if True: # settings.LINO.user_model:
         
         @classmethod
         def get_request_queryset(cls,rr):
-            #~ logger.info("20120114 param_values = %r",rr.param_values)
+            #~ logger.info("20120608.get_request_queryset param_values = %r",rr.param_values)
             qs = super(ContractsSearch,cls).get_request_queryset(rr)
             #~ user = rr.param_values.get('user',None)
             if rr.param_values.user:
                 qs = qs.filter(user=rr.param_values.user)
             if rr.param_values.type:
                 qs = qs.filter(type=rr.param_values.type)
-            today = rr.param_values.today
+            today = rr.param_values.today or datetime.date.today()
             #~ today = rr.param_values.get('today',None) or datetime.date.today()
             #~ show_active = rr.param_values.get('show_active',True)
             if today:
@@ -1069,26 +1199,33 @@ if True: # settings.LINO.user_model:
 
 COLS = 8
 
-class JobsOverview(dd.EmptyTable):
+
+class JobsOverview(mixins.EmptyTable):
+    """
+    """
+    required = dict(user_groups=['integ'])
     label = _("Contracts Situation") 
+    #~ detail_layout = JobsOverviewDetail()
+    detail_template = "body"
     
     parameters = dict(
-      date = models.DateField(default=datetime.date.today),
+      date = models.DateField(default=datetime.date.today,blank=True,null=True),
       contract_type = models.ForeignKey(ContractType,blank=True,null=True),
       job_type = models.ForeignKey(JobType,blank=True,null=True),
       )
     params_panel_hidden = True
 
     @dd.displayfield(_("Body"))
-    def body(cls,self,req):
+    def body(cls,self,ar):
+        #~ logger.info("20120221 3 body(%s)",req)
         #~ logger.info("Waiting 5 seconds...")
         #~ time.sleep(5)
         today = self.date or datetime.date.today()
         html = ''
         rows = []
           
-        if self.job_type:
-            jobtypes = [self.job_type]
+        if ar.param_values.job_type:
+            jobtypes = [ar.param_values.job_type]
         else:
             jobtypes = JobType.objects.all()
         for jobtype in jobtypes:
@@ -1097,17 +1234,21 @@ class JobsOverview(dd.EmptyTable):
             for job in jobtype.job_set.order_by('provider'):
                 actives = []
                 candidates = []
-                qs = job.contract_set.all()
-                if self.contract_type:
-                    qs = qs.filter(type=self.contract_type)
+                #~ qs = job.contract_set.all()
+                qs = job.contract_set.order_by('applies_from')
+                if ar.param_values.contract_type:
+                    qs = qs.filter(type=ar.param_values.contract_type)
                 for ct in qs:
                     if ct.applies_from:
                         until = ct.date_ended or ct.applies_until
                         if not until or (ct.applies_from <= today and until >= today):
                             actives.append(ct)
-                for req in job.candidature_set.all():
+                qs = job.candidature_set.order_by('date_submitted')
+                qs = pcsw.only_coached_persons(qs,today,
+                    'person__coached_from','person__coached_until')
+                for cand in qs:
                     #~ if not req.contract:
-                    candidates.append(req)
+                    candidates.append(cand)
                 if candidates + actives:
                     s = "<p>"
                     s += "<b>%s (%s)</b>" % (
@@ -1144,71 +1285,11 @@ class JobsOverview(dd.EmptyTable):
         return html
 
 
-class ContractsSituation(mixins.Listing):
-    #~ template_name = 'jobs/ContractsSituation/Listing.odt'
-    build_method = 'appyrtf'
     
-    class Meta:
-        verbose_name = _("Contracts Situation") 
-        
-    contract_type = models.ForeignKey(ContractType,blank=True,null=True)
-    job_type = models.ForeignKey(JobType,blank=True,null=True)
-    
-    def body(self):
-        today = self.date or datetime.date.today()
-        html = ''
-        rows = []
-          
-        for jobtype in JobType.objects.all():
-            cells = []
-            #~ for job in jobtype.job_set.all():
-            for job in jobtype.job_set.order_by('provider'):
-                actives = []
-                candidates = []
-                for ct in job.contract_set.all():
-                    if ct.applies_from:
-                        until = ct.date_ended or ct.applies_until
-                        if not until or (ct.applies_from <= today and until >= today):
-                            actives.append(ct)
-                for req in job.candidature_set.all():
-                    #~ if not req.contract:
-                    candidates.append(req)
-                if candidates + actives:
-                    s = "<p>"
-                    s += "<b>%s (%s)</b>" % (
-                      cgi.escape(unicode(job)),job.capacity)
-                    if job.remark:
-                        s += " <i>%s</i>" % cgi.escape(job.remark)
-                    s += "</p>"
-                    s += UL([u'%s bis %s' % (
-                      ct.person.last_name.upper(),
-                      babel.dtos(ct.applies_until)
-                    ) for ct in actives])
-                    #~ s += "<li>"
-                    #~ for ct in actives:
-                        #~ s += '<li>%s</li>' % cgi.escape(unicode(ct.person))
-                    #~ s += "</li>"
-                    if candidates:
-                        s += "<p>%s:</p>" % cgi.escape(_("Candidates"))
-                        s += UL([i.person for i in candidates])
-                        #~ for ct in candidates:
-                            #~ s += '<br>' + cgi.escape(unicode(ct.person))
-                    cells.append(s)
-            if cells:
-                html += '<h1>%s</h1>' % cgi.escape(unicode(jobtype))
-                #~ head = ''.join(['<col width="30" />' for c in cells])
-                #~ head = '<colgroup>%s</colgroup>' % head
-                s = ''.join(['<td valign="top">%s</td>' % c for c in cells])
-                s = '<tr>%s</tr>' % s
-                #~ s = head + s
-                html += '<table border="1" width="100%%">%s</table>' % s
-        html = '<div class="htmlText">%s</div>' % html
-        return html
-
 
 if True: # dd.is_installed('contacts') and dd.is_installed('jobs'):
   
-    dd.inject_field(Company,
+    dd.inject_field(pcsw.Company,
         'is_jobprovider',
         mti.EnableChild('jobs.JobProvider',verbose_name=_("is Job Provider")),
         """Whether this Company is also a Job Provider."""
@@ -1216,33 +1297,38 @@ if True: # dd.is_installed('contacts') and dd.is_installed('jobs'):
 
 
 def setup_main_menu(site,ui,user,m): 
-    if user.is_spis:
-        m = m.add_menu("jobs",_("Jobs"))
-        m.add_action(JobProviders)
-        m.add_action(Jobs)
-        m.add_action(Offers)
-        m.add_action(ContractsSearch)
+    #~ if user.profile.integ_level < UserLevels.user:
+        #~ return
+    m = m.add_menu("jobs",_("Jobs"))
+    m.add_action(JobProviders)
+    m.add_action(Jobs)
+    m.add_action(Offers)
+    m.add_action(ContractsSearch)
 
 def setup_my_menu(site,ui,user,m): 
-    if user.is_spis:
-        m.add_action(MyContracts)
+    #~ if user.profile.integ_level < UserLevels.user:
+        #~ return
+    m.add_action(MyContracts)
   
 def setup_config_menu(site,ui,user,m): 
-    if user.is_spis:
-        m  = m.add_menu("jobs",_("~Jobs"))
-        m.add_action(ContractTypes)
-        m.add_action(JobTypes)
-        m.add_action(Sectors)
-        m.add_action(Functions)
-        m.add_action(StudyTypes)
-        m.add_action(Schedules)
-        m.add_action(Regimes)
+    #~ if user.profile.integ_level < UserLevels.manager:
+        #~ return
+    m  = m.add_menu("jobs",_("~Jobs"))
+    m.add_action(ContractTypes)
+    m.add_action(JobTypes)
+    m.add_action(Sectors)
+    m.add_action(Functions)
+    m.add_action(StudyTypes)
+    m.add_action(Schedules)
+    m.add_action(Regimes)
             
     
     
   
 def setup_explorer_menu(site,ui,user,m):
-    if user.is_spis:
-        m.add_action(Contracts)
-        m.add_action(Candidatures)
-        m.add_action(Studies)
+    #~ if user.profile.integ_level < UserLevels.manager:
+        #~ return
+    m  = m.add_menu("jobs",_("~Jobs"))
+    m.add_action(Contracts)
+    m.add_action(Candidatures)
+    m.add_action(Studies)
